from typing import Callable
import torch
import math
import os
import numpy as np
from tqdm.auto import tqdm
import torch.nn.functional as F
from diffusers.loaders import AttnProcsLayers
from diffusers.optimization import get_scheduler
from ailab.atp_dataset.dataset import AILabDataset
from ailab.atp_finetuner.trainer import AILabTrainer 
from ailab.atp_finetuner.model import AILabModel
from ailab.atp_finetuner.datacollator import AILabDataCollator
from ailab.atp_finetuner.metric import AILabMetric
from ailab.atp_finetuner.preprossor import AILabPreprocessor
from ailab.atp_finetuner.build import TrainerRg
from ailab.atp_finetuner.constant import Task, Model
from ailab.log import logger

@TrainerRg.register((Task.text_to_image, Model.stable_diffusion_2_1))
class SDTrainer(AILabTrainer):
    def __init__(self):
        super().__init__()

    def preprocess(self, dataset:AILabDataset, model:AILabModel, preprocessor: AILabPreprocessor, \
                      data_collator:AILabDataCollator, metric:AILabMetric, train_progress:Callable, **kwargs):
        train_args = kwargs['train_args']
        self.train_args = train_args
        self.dataset = dataset
        self.model = model
        self.train_progress = train_progress
    
    def train(self):
        weight_dtype = torch.float16
        dataloader_num_workers = 8
        lr_warmup_steps = 0
        max_train_steps = 5000
        lr_scheduler = "cosine"
        train_batch_size = 1
        validation_epochs = 1
        validation_prompt = "Totoro"
        max_grad_norm = 1
        checkpointing_steps = 500
        num_validation_images = 4
        train_args = self.train_args
        model = self.model
        dataset = self.dataset

        learning_rate = train_args.get('learning_rate', 1e-5)
        gradient_accumulation_steps = train_args.get('gradient_accumulation_steps', 4)
        output_dir = train_args.get('output_dir', "./my_sd_model")
        resume_from_checkpoint = train_args.get('resume_from_checkpoint', False)
        text_encoder = model.get_inside_models('text_encoder')
        unet = model.get_inside_models('unet')
        vae = model.get_inside_models('vae')
        noise_scheduler = model.get_inside_models('noise_scheduler')
        train_dataset = dataset.to_hf_dataset()
        accelerator = self.accelerator.accelerator_ins

        lora_layers = AttnProcsLayers(unet.attn_processors)
        optimizer_cls = torch.optim.AdamW
        optimizer = optimizer_cls(
            lora_layers.parameters(),
            lr=learning_rate,
            betas=(0.9, 0.999),
            weight_decay=1e-2,
            eps=1e-08,
        )

        def collate_fn(examples):
            pixel_values = torch.stack([example["pixel_values"] for example in examples])
            pixel_values = pixel_values.to(memory_format=torch.contiguous_format).float()
            input_ids = torch.stack([example["input_ids"] for example in examples])
            return {"pixel_values": pixel_values, "input_ids": input_ids}

        # DataLoaders creation:
        train_dataloader = torch.utils.data.DataLoader(
            train_dataset,
            shuffle=True,
            collate_fn=collate_fn,
            batch_size=train_batch_size,
            num_workers=dataloader_num_workers,
        )

        # Scheduler and math around the number of training steps.
        num_update_steps_per_epoch = math.ceil(len(train_dataloader) / gradient_accumulation_steps)

        lr_scheduler = get_scheduler(
            lr_scheduler,
            optimizer=optimizer,
            num_warmup_steps=lr_warmup_steps * gradient_accumulation_steps,
            num_training_steps=max_train_steps * gradient_accumulation_steps,
        )

        # Prepare everything with our `accelerator`.
        lora_layers, optimizer, train_dataloader, lr_scheduler = accelerator.prepare(
            lora_layers, optimizer, train_dataloader, lr_scheduler
        )

        # We need to recalculate our total training steps as the size of the training dataloader may have changed.
        num_update_steps_per_epoch = math.ceil(len(train_dataloader) / gradient_accumulation_steps)
        # Afterwards we recalculate our number of training epochs
        num_train_epochs = math.ceil(max_train_steps / num_update_steps_per_epoch)

        # We need to initialize the trackers we use, and also store our configuration.
        # The trackers initializes automatically on the main process.
        if accelerator.is_main_process:
            accelerator.init_trackers("text2image-fine-tune")
        

        total_batch_size = train_batch_size * accelerator.num_processes * gradient_accumulation_steps

        logger.info("***** Running training *****")
        logger.info(f"  Num examples = {len(train_dataset)}")
        logger.info(f"  Num Epochs = {num_train_epochs}")
        logger.info(f"  Instantaneous batch size per device = {train_batch_size}")
        logger.info(f"  Total train batch size (w. parallel, distributed & accumulation) = {total_batch_size}")
        logger.info(f"  Gradient Accumulation steps = {gradient_accumulation_steps}")
        logger.info(f"  Total optimization steps = {max_train_steps}")
        global_step = 0
        first_epoch = 0

        if resume_from_checkpoint:
            # Get the most recent checkpoint
            # Get the most recent checkpoint
            dirs = os.listdir(output_dir)
            dirs = [d for d in dirs if d.startswith("checkpoint")]
            dirs = sorted(dirs, key=lambda x: int(x.split("-")[1]))
            path = dirs[-1] if len(dirs) > 0 else None

            if path is None:
                resume_from_checkpoint = None
            else:
                logger.info(f"Resuming from checkpoint {resume_from_checkpoint}")
                accelerator.load_state(os.path.join(output_dir, path))
                global_step = int(path.split("-")[1])

                resume_global_step = global_step * gradient_accumulation_steps
                first_epoch = global_step // num_update_steps_per_epoch
                resume_step = resume_global_step % (num_update_steps_per_epoch * gradient_accumulation_steps)

        # Only show the progress bar once on each machine.
        progress_bar = tqdm(range(global_step, max_train_steps), disable=not accelerator.is_local_main_process)
        progress_bar.set_description("Steps")

        for epoch in range(first_epoch, num_train_epochs):
            unet.train()
            train_loss = 0.0
            for step, batch in enumerate(train_dataloader):
                if resume_from_checkpoint and epoch == first_epoch and step < resume_step:
                    continue

                with accelerator.accumulate(unet):
                    # Convert images to latent space
                    latents = vae.encode(batch["pixel_values"].to(dtype=weight_dtype)).latent_dist.sample()
                    latents = latents * vae.config.scaling_factor

                    # Sample noise that we'll add to the latents
                    noise = torch.randn_like(latents)

                    bsz = latents.shape[0]
                    # Sample a random timestep for each image
                    timesteps = torch.randint(0, noise_scheduler.config.num_train_timesteps, (bsz,), device=latents.device)
                    timesteps = timesteps.long()

                    # Add noise to the latents according to the noise magnitude at each timestep
                    # (this is the forward diffusion process)
                    noisy_latents = noise_scheduler.add_noise(latents, noise, timesteps)

                    # Get the text embedding for conditioning
                    encoder_hidden_states = text_encoder(batch["input_ids"])[0]

                    # Get the target for loss depending on the prediction type
                    if noise_scheduler.config.prediction_type == "epsilon":
                        target = noise
                    elif noise_scheduler.config.prediction_type == "v_prediction":
                        target = noise_scheduler.get_velocity(latents, noise, timesteps)
                    else:
                        raise ValueError(f"Unknown prediction type {noise_scheduler.config.prediction_type}")

                    # Predict the noise residual and compute loss
                    model_pred = unet(noisy_latents, timesteps, encoder_hidden_states).sample
                    loss = F.mse_loss(model_pred.float(), target.float(), reduction="mean")

                    # Gather the losses across all processes for logging (if we use distributed training).
                    avg_loss = accelerator.gather(loss.repeat(train_batch_size)).mean()
                    train_loss += avg_loss.item() / gradient_accumulation_steps

                    # Backpropagate
                    accelerator.backward(loss)
                    if accelerator.sync_gradients:
                        params_to_clip = lora_layers.parameters()
                        accelerator.clip_grad_norm_(params_to_clip, max_grad_norm)
                    optimizer.step()
                    lr_scheduler.step()
                    optimizer.zero_grad()

                # Checks if the accelerator has performed an optimization step behind the scenes
                if accelerator.sync_gradients:
                    progress_bar.update(1)
                    global_step += 1
                    self.train_progress(global_step * 100 / max_train_steps)
                    accelerator.log({"train_loss": train_loss}, step=global_step)
                    train_loss = 0.0

                    if global_step % checkpointing_steps == 0:
                        if accelerator.is_main_process:
                            save_path = os.path.join(output_dir, f"checkpoint-{global_step}")
                            accelerator.save_state(save_path)
                            logger.info(f"Saved state to {save_path}")

                logs = {"step_loss": loss.detach().item(), "lr": lr_scheduler.get_last_lr()[0]}
                progress_bar.set_postfix(**logs)

                if global_step >= max_train_steps:
                    break

            if accelerator.is_main_process:
                torch.cuda.empty_cache()

        # Save the lora layers
        accelerator.wait_for_everyone()
        if accelerator.is_main_process:
            unet = unet.to(torch.float32)
            unet.save_attn_procs(output_dir)

    def postprocess(self):
        pass






from transformers.data import data_collator
from ailab.atp_finetuner.datacollator import AILabDataCollator
from ailab.atp_finetuner.build import DataCollatorRg
from ailab.atp_finetuner import constant 
from ailab.atp_finetuner.preprossor import AILabPreprocessor
from ailab.atp_finetuner.model import AILabModel
from ailab.atp_finetuner.constant import Task, Model

@DataCollatorRg.register((Task.question_answering_cn, Model.vicuna))
@DataCollatorRg.register((Task.question_answering_cn, Model.open_llama))
class DataCollatorForLanguage(AILabDataCollator) :
    def __init__(self, datacollator, preprocessor):
        super().__init__(datacollator, preprocessor)

    def forward(self, **kwargs):
        pass
    
    @classmethod
    def build_datacollator(cls, framework:constant.Framework, preprocessor:AILabPreprocessor, model:AILabModel) :
        datacollator = data_collator.DataCollatorForLanguageModeling(
            preprocessor.preprocessor_ins, mlm=False
        )
        return cls(datacollator, preprocessor)


#!/usr/bin/env python
# coding:utf-8
"""
@license: Apache License2
@author: hanxiao
@time: 2023/03/20
@project: ailab
"""
import json
from aiges.core.types import *
# from ifly_atp_sdk.huggingface.pipelines import pipeline
from transformers import pipeline

try:
    from aiges_embed import ResponseData, Response, DataListNode, DataListCls  # c++
except:
    from aiges.dto import Response, ResponseData, DataListNode, DataListCls

from aiges.sdk import WrapperBase, \
    AudioBodyField, ImageBodyField, \
    StringBodyField, StringParamField
from aiges.utils.log import log, getFileLogger

task = "feature-extraction"
model = "facebook/bart-base"
input1_key = "text"


# 定义模型的超参数和输入参数
class UserRequest(object):
    input1 = StringBodyField(key=input1_key, value="I like eating fish!".encode("utf-8"))


# 定义模型的输出参数
class UserResponse(object):
    accept1 = StringBodyField(key="result")


# 定义服务推理逻辑
class Wrapper(WrapperBase):
    serviceId = task
    version = "v1"
    requestCls = UserRequest()
    responseCls = UserResponse()
    model = None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.filelogger = None

    def wrapperInit(self, config: {}) -> int:
        log.info("Initializing ...")
        self.pipe = pipeline(task=task, model=model)
        self.filelogger = getFileLogger()
        return 0

    def wrapperOnceExec(self, params: {}, reqData: DataListCls) -> Response:
        self.filelogger.info("got reqdata , %s" % reqData.list)
        input_text = reqData.get(input1_key).data.decode("utf-8")
        result = self.pipe(input_text)
        self.filelogger.info(result)

        # 使用Response封装result
        res = Response()
        resd = ResponseData()
        resd.key = "result"
        resd.setDataType(DataText)
        resd.status = Once
        resd.setData(json.dumps(result).encode("utf-8"))
        res.list = [resd]
        return res

    def wrapperFini(cls) -> int:
        return 0

    def wrapperError(cls, ret: int) -> str:
        if ret == 100:
            return "user error defined here"
        return ""

    '''
        此函数保留测试用，不可删除
    '''

    def wrapperTestFunc(cls, data: [], respData: []):
        pass


if __name__ == '__main__':
    m = Wrapper(legacy=False)
    m.run()
    # print(m.schema())



from typing import Callable
import math
import os
from transformers import TrainingArguments, Trainer
from peft import get_peft_model_state_dict
from ailab.atp_dataset.dataset import AILabDataset
from ailab.atp_finetuner.trainer import AILabTrainer 
from ailab.atp_finetuner.model import AILabModel
from ailab.atp_finetuner.datacollator import AILabDataCollator
from ailab.atp_finetuner.metric import AILabMetric
from ailab.atp_finetuner.preprossor import AILabPreprocessor
from ailab.atp_finetuner.build import TrainerRg
from ailab.atp_finetuner.constant import Task, Model
from ailab.utils.callbacks import TrainProgress
from ailab.log import logger

@TrainerRg.register((Task.question_answering, Model.chinese_alpaca))
class ChineseAlpacaTrainer(AILabTrainer):
    def __init__(self):
        super().__init__()

    def preprocess(self, dataset:AILabDataset, model:AILabModel, preprocessor: AILabPreprocessor, \
                      data_collator:AILabDataCollator, metric:AILabMetric, train_progress:Callable, **kwargs):
        train_args = kwargs['train_args']
        output_dir = train_args.get('output_dir', 'my_model')
        learning_rate = train_args.get('learning_rate', 1e-5)
        num_train_epochs = train_args.get('num_train_epochs', 2)
        evaluation_strategy = train_args.get('evaluation_strategy', "epoch")
        save_strategy = train_args.get('save_strategy', "steps")
        per_device_train_batch_size = train_args.get('per_device_train_batch_size', 4)
        gradient_accumulation_steps = train_args.get('gradient_accumulation_steps', 1)
        logging_steps = train_args.get('logging_steps', 10)
        warmup_steps = train_args.get('warmup_steps', 0)
        fp16 = train_args.get('fp16', True)
        bf16 = train_args.get('bf16', False)
        eval_steps = train_args.get('eval_steps', 250)
        save_steps = train_args.get('save_steps', 500)
        resume_from_checkpoint = train_args.get('resume_from_checkpoint', False)

        deepspeed_dir = os.path.dirname(os.path.abspath(__file__))
        deepspeed_dir = os.path.join(deepspeed_dir,"ds_zero2_no_offload.json")
        training_args=TrainingArguments(
            per_device_train_batch_size=per_device_train_batch_size,
            per_device_eval_batch_size=4,
            gradient_accumulation_steps=gradient_accumulation_steps,
            lr_scheduler_type="cosine",
            warmup_ratio=0.03,
            warmup_steps=0,
            weight_decay=0.0,
            learning_rate=learning_rate,
            fp16=fp16,
            bf16=bf16,
            logging_steps=logging_steps,
            evaluation_strategy=evaluation_strategy,
            save_strategy=save_strategy,
            eval_steps=eval_steps,
            save_steps=save_steps,
            output_dir=output_dir,
            save_total_limit=3,
            load_best_model_at_end=False,
            group_by_length=False,
            report_to= None,
            run_name= None,
            push_to_hub=False,
            num_train_epochs=num_train_epochs,
            ddp_find_unused_parameters=False,
            ddp_timeout=30000,
            deepspeed=deepspeed_dir,
            do_train=True,
            gradient_checkpointing=True,
            greater_is_better=None,
        )

        logger.info(f'ChineseAlpacaTrainer training_args {training_args}')

        model = model.model_ins
        tokenizer = preprocessor.preprocessor_ins
        embedding_size = model.get_input_embeddings().weight.shape[0]
        if len(tokenizer) != embedding_size:
            logger.info("resize the embedding size by the size of the tokenizer")
            model.resize_token_embeddings(len(tokenizer))

        lora_rank=8
        lora_alpha=32
        lora_trainable="q_proj,v_proj,k_proj,o_proj,gate_proj,down_proj,up_proj"
        modules_to_save="embed_tokens,lm_head"
        lora_dropout=0.05

        from peft import LoraConfig, TaskType, get_peft_model
        target_modules = lora_trainable.split(',')
        modules_to_save = modules_to_save.split(',')
        peft_config = LoraConfig(
            task_type=TaskType.CAUSAL_LM, 
            target_modules=target_modules,
            inference_mode=False, 
            r=lora_rank, lora_alpha=lora_alpha, 
            lora_dropout=lora_dropout,
            modules_to_save=modules_to_save)
        model = get_peft_model(model, peft_config)

        #model.base_model.tie_weights()
        model.print_trainable_parameters()
        logger.info(f"model.modules_to_save: {model.modules_to_save}")
        old_state_dict = model.state_dict
        model.state_dict = (
            lambda self, *_, **__: get_peft_model_state_dict(self, old_state_dict())
        ).__get__(model, type(model))

        trainer = Trainer(
            model=model,
            args=training_args,
            train_dataset=dataset.to_hf_dataset()["train"],
            eval_dataset=dataset.to_hf_dataset()["test"],
            data_collator=data_collator.datacollator_ins,
            callbacks=[TrainProgress(train_progress)],
        )

        self.trainer = trainer
        self.model = model
        self.tokenizer = tokenizer
        self.output_dir = output_dir
        self.resume_from_checkpoint = resume_from_checkpoint
    
    def train(self):
        model = self.model
        tokenizer = self.tokenizer
        trainer = self.trainer
        output_dir = self.output_dir 
        resume_from_checkpoint = self.resume_from_checkpoint

        train_result = trainer.train(resume_from_checkpoint=resume_from_checkpoint)
        trainer.save_model()  # Saves the tokenizer too for easy upload

        metrics = train_result.metrics
        metrics["train_samples"] = len(trainer.train_dataset)
        trainer.log_metrics("train", metrics)
        trainer.save_metrics("train", metrics)
        trainer.save_state()

        import torch.distributed as dist
        if dist.get_rank() == 0:
            import shutil
            from transformers.modeling_utils import unwrap_model
            try:
                unwrap_model(model).peft_config.save_pretrained(output_dir)
            except AttributeError:
                unwrap_model(model).peft_config['default'].save_pretrained(output_dir)
            shutil.move(
                os.path.join(output_dir,'pytorch_model.bin'),
                os.path.join(output_dir,'adapter_model.bin'))
            tokenizer.save_pretrained(output_dir)

        logger.info("*** Evaluate ***")
        metrics = trainer.evaluate()
        metrics["eval_samples"] =len(trainer.eval_dataset)
        try:
            perplexity = math.exp(metrics["eval_loss"])
        except OverflowError:
            perplexity = float("inf")
        metrics["perplexity"] = perplexity

        trainer.log_metrics("eval", metrics)
        trainer.save_metrics("eval", metrics)

    def postprocess(self):
        self.trainer.evaluate()






#!/usr/bin/env python
# coding:utf-8
"""
@license: Apache License2
@file: wrapper.py
@time: 2022-08-19 02:05:07.467170
@project: mnist
@project: ./
"""
import json
import torch
import os.path
from aiges.core.types import *
try:
    from aiges_embed import ResponseData, Response, DataListNode, DataListCls  # c++
except:
    from aiges.dto import Response, ResponseData, DataListNode, DataListCls

from aiges.sdk import WrapperBase, \
    ImageBodyField, \
    StringBodyField, StringParamField
from aiges.utils.log import log, getFileLogger
from ailab.log import logger

# 定义模型的超参数和输入参数
class UserRequest(object):
    input1 = StringBodyField(key="text", value=b"I have a problem with my iphone that needs to be resolved asap!!")


# 定义模型的输出参数
class UserResponse(object):
    accept1 = StringBodyField(key="result")


# 定义服务推理逻辑
class Wrapper(WrapperBase):
    serviceId = "chatglm2_6b"
    version = "v1"
    requestCls = UserRequest()
    responseCls = UserResponse()
    model = None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.filelogger = None

    def wrapperInit(self, config: {}) -> int:
        logger.info("Initializing ...")
        from transformers import AutoTokenizer,AutoConfig,AutoModel
        base_model = os.environ.get("PRETRAINED_MODEL_NAME")
        lora_weight = os.environ.get("MODEL_PATH")
        tokenizer_path = os.environ.get("TOKENIZER_PATH")
        if not base_model or not lora_weight or not tokenizer_path:
            log.error("should have environ(PRETRAINED_MODEL_NAME,MODEL_PATH(lora weight dir）,TOKENIZER_PATH)")
            return -1
        
        from peft import PeftModel
        tokenizer = AutoTokenizer.from_pretrained(
            tokenizer_path,
            use_fast=False,
            padding_side="left",
            trust_remote_code=True
        )

        config = AutoConfig.from_pretrained(base_model,trust_remote_code=True)
        model = AutoModel.from_pretrained(base_model, config=config, 
                                            trust_remote_code=True, device_map="auto")
        assert tokenizer.eos_token_id is not None, "Please update the *.json and *.py files of ChatGLM2-6B from HuggingFace."
        model.lm_head = model.transformer.output_layer
        model = PeftModel.from_pretrained(model, lora_weight)
        model.requires_grad_(False) # fix all model params
        model = model.half() # cast all params to float16 for inference
        model = model.cuda()
        model.eval()
        self.model = model
        self.tokenizer = tokenizer
        self.filelogger = getFileLogger()
        return 0

    def wrapperOnceExec(self, params: {}, reqData: DataListCls) -> Response:
        self.filelogger.info("got reqdata , %s" % reqData.list)
        tokenizer = self.tokenizer
        model = self.model
        instruction = reqData.get("text").data.decode('utf-8')

        history = []
        generating_args = {
            "do_sample":True,
            "temperature":0.95,
            "top_p":0.7,
            "top_k":50,
            "num_beams":1,
            "max_length":2048,
            "max_new_tokens":None,
            "repetition_penalty":1.0,
        }
        for _, history in model.stream_chat(tokenizer, instruction, history=history, **generating_args):
            pass
        for query, response in history:
            pass
        result = response
        self.filelogger.info("got result , %s" % result)
        # 使用Response封装result
        res = Response()
        resd = ResponseData()
        resd.key = "result"
        resd.setDataType(DataText)
        resd.status = Once
        resd.setData(result.encode("utf-8"))
        res.list = [resd]
        return res

    def wrapperFini(cls) -> int:
        return 0

    def wrapperError(cls, ret: int) -> str:
        if ret == 100:
            return "user error defined here"
        return ""

    '''
        此函数保留测试用，不可删除
    '''

    def wrapperTestFunc(cls, data: [], respData: []):
        pass


if __name__ == '__main__':
    m = Wrapper()
    m.run()



import datetime
import time
from typing import Optional
from ailab.atp_evaluation.constant import Task, Model, BenchMarkType
from ailab.atp_evaluation.models import AILabModel
from ailab.atp_evaluation.benchmarks import AILabBenchmark
from ailab.atp_evaluation.benchmarks.harness import HarnessBenchmark
from ailab.log import logger

class AILabEvaluator:
    def __init__(self, benchmark_type: BenchMarkType,
                task: Task = None,
                model_name: Model = None,
                dataset_dir: str = None,
                ntrain: int = 0,
                model_dir: Optional[str] = None,
                tokenizer_dir: Optional[str] = None,
                lora_weight_dir: Optional[str] = None,
                output_dir: Optional[str] = ".",
                **args):
        if benchmark_type == BenchMarkType.harness:
            harness_args = args['harness_args']
            benchmark = HarnessBenchmark(**harness_args)
        else:
            model = AILabModel.from_pretrained('cuda', task, model_name, model_dir, lora_weight_dir, tokenizer_dir, **args)
            benchmark = AILabBenchmark.from_project_config(model, benchmark_type, dataset_dir, ntrain, output_dir, **args)
        self._benchmark = benchmark

    def evaluate(self):
        start_time = time.time()
        self._benchmark.evaluate()
        end_time = time.time()
        delta_time = datetime.timedelta(seconds=(end_time-start_time))
        logger.info(f"finished, cost time {str(delta_time)}")



import torch
from ailab.atp_finetuner.preprossor import AILabPreprocessor
from ailab.atp_dataset.dataset import AILabDataset
from transformers import LlamaTokenizer
from ailab.atp_finetuner.build import PreProcessorRg
from ailab.atp_finetuner.constant import Task, Model


@PreProcessorRg.register((Task.question_answering, Model.chinese_alpaca))
class ChineseAlpacaPreProcessor(AILabPreprocessor):
    def __init__(self, dataset, preprocessor):
        super().__init__(dataset, preprocessor)

    @classmethod
    def build_preprocessor(cls, model_name:str, dataset: AILabDataset, pc_dir:str, **kwargs):
        pc_name_dir = model_name if pc_dir is None else pc_dir
        tokenizer = LlamaTokenizer.from_pretrained(pc_name_dir, legacy=False)
        return cls(dataset, tokenizer)

    def process_data(self) ->AILabDataset:
        tokenizer = self._preprocessor
        dataset = self._dataset.to_hf_dataset()

        def buid_instruction_dataset(max_seq_length: int, preprocessing_num_workers = None,):
            from datasets import concatenate_datasets
            PROMPT_TEMPLATE = (
                "Below is an instruction that describes a task. "
                "Write a response that appropriately completes the request.\n\n"
                "### Instruction:\n{instruction}\n\n### Response: "
            )
            IGNORE_INDEX = -100

            def tokenization(examples):
                sources = []
                targets = []
                prompt = PROMPT_TEMPLATE
                for instruction, input, output in zip(examples['instruction'],examples['input'],examples['output']):
                    if input is not None and input !="":
                        instruction = instruction+'\n'+input
                    source = prompt.format_map({'instruction':instruction})
                    target = f"{output}{tokenizer.eos_token}"

                    sources.append(source)
                    targets.append(target)

                tokenized_sources = tokenizer(sources,return_attention_mask=False)
                tokenized_targets = tokenizer(targets,return_attention_mask=False,add_special_tokens=False)

                all_input_ids = []
                all_labels = []
                for s,t in zip(tokenized_sources['input_ids'],tokenized_targets['input_ids']):
                    input_ids = torch.LongTensor(s + t)[:max_seq_length]
                    labels = torch.LongTensor([IGNORE_INDEX] * len(s) + t)[:max_seq_length]
                    assert len(input_ids) == len(labels)
                    all_input_ids.append(input_ids)
                    all_labels.append(labels)

                results = {'input_ids':all_input_ids, 'labels': all_labels}
                return results

            all_datasets = []
            tokenization_func = tokenization
            tokenized_dataset = dataset.map(
                tokenization_func,
                batched=True,
                num_proc=preprocessing_num_workers,
                remove_columns=["instruction","input","output"],
                keep_in_memory=False,
                desc="preprocessing on dataset",
            )
            processed_dataset = tokenized_dataset
            processed_dataset.set_format('torch')
            all_datasets.append(processed_dataset['train'])
            all_datasets = concatenate_datasets(all_datasets)
            return processed_dataset

        max_seq_length = 512
        preprocessing_num_workers = 8
        tokenized_dataset = buid_instruction_dataset(max_seq_length,preprocessing_num_workers)
        return AILabDataset(tokenized_dataset)



#!/usr/bin/env python
# coding:utf-8
"""
@license: Apache License2
@file: wrapper.py
@time: 2023-03-02 18:58:48.583182
@project: yolo
@project: ./
"""

import sys
import hashlib
import json
from aiges.core.types import *
from aiges.dto import Response, ResponseData, DataListNode, DataListCls

from aiges.sdk import WrapperBase, \
    StringParamField, \
    ImageBodyField, \
    StringBodyField
from aiges.utils.log import log

########
# 请在此区域导入您的依赖库

# Todo
# for example: import numpy
#import os
#import sys
#import io

import argparse
import platform
import torch
import base64
import numpy as np
from pathlib import Path
from models.common import DetectMultiBackend
from utils.dataloaders import IMG_FORMATS, VID_FORMATS, LoadImages, LoadScreenshots, LoadStreams
from utils.general import (LOGGER, Profile, check_file, check_img_size, check_imshow, check_requirements, colorstr, cv2,
                           increment_path, non_max_suppression, print_args, scale_boxes, strip_optimizer, xyxy2xywh)
from utils.augmentations import (Albumentations, augment_hsv, classify_albumentations, classify_transforms, copy_paste,
                                 letterbox, mixup, random_perspective)
from utils.plots import Annotator, colors, save_one_box
from utils.torch_utils import select_device, smart_inference_mode
#要记得把 models utils 这两个文件夹拷过去

########


# 定义模型的超参数和输入参数
class UserRequest(object):
    input1 = ImageBodyField(key="img", path="test_data/bus.jpg")


class UserResponse(object):
    accept1 = StringBodyField(key="img")

class Wrapper(WrapperBase):
    serviceId = "yolo"
    version = "v1"
    requestCls = UserRequest()
    responseCls = UserResponse()
    model = None

    #加载模型
    def wrapperInit(cls, config: {}) -> int:
        log.info(config)
        log.info("Initializing ...")
        device = select_device('cpu')
        cls.model = DetectMultiBackend('./yolov5s.pt', device=device, dnn=False, data=False, fp16=False)
        return 0

    def wrapperOnceExec(cls, params: {}, reqData: DataListCls) -> Response:
        #log.info("got reqdata , %s" % reqData.list)
        for req in reqData.list:
            log.info("reqData key: %s , size is %d" % (req.key, len(req.data)))
        log.warning("reqData bytes md5sum is %s" % hashlib.md5(reqData.list[0].data).hexdigest())
        log.info("I am infer logic...please inplement")
        log.info("Testing reqData get: ")
        rg = reqData.get("img")
        log.info("get key: %s" % rg.key)
        log.info("get key: %d" % len(rg.data))

        # read pic
        stride, names, pt = cls.model.stride, cls.model.names, cls.model.pt
        imagebytes = reqData.get("img").data
        #img = base64.b64decode(imagebytes)
        imgarr = np.frombuffer(imagebytes, np.uint8)  # 转换np序列
        im0s = cv2.imdecode(imgarr, cv2.IMREAD_COLOR) # change to opencv BGR
        im = letterbox(im0s, 640, stride=stride, auto=pt)[0]  # padded resize
        im = im.transpose((2, 0, 1))[::-1]  # HWC to CHW, BGR to RGB
        im = np.ascontiguousarray(im)  # contiguous

        # inference
        im = torch.from_numpy(im).to(cls.model.device)
        im = im.float()  # uint8 to fp16/32
        im /= 255  # 0 - 255 to 0.0 - 1.0
        if len(im.shape) == 3:
            im = im[None]  # expand for batch dim
        pred = cls.model(im, augment=False, visualize=False)
        pred = non_max_suppression(pred, 0.25, 0.45, None, False, max_det=1000)

        for i, det in enumerate(pred):  # per image
            im0 = im0s.copy()
            gn = torch.tensor(im0.shape)[[1, 0, 1, 0]]  # normalization gain whwh
            imc = im0
            annotator = Annotator(im0, line_width=3, example=str(names))
            if len(det):
                # Rescale boxes from img_size to im0 size
                det[:, :4] = scale_boxes(im.shape[2:], det[:, :4], im0.shape).round()
                for *xyxy, conf, clas in reversed(det):
                    c = int(clas)  # integer class
                    label = names[c]
                    annotator.box_label(xyxy, label, color=colors(c, True))
        im0 = annotator.result()

        # response
        img_base64 = base64.b64encode(im0)
        result_msg = {
            "img": "the result is: %s" % img_base64
        }
        res = Response()
        resd = ResponseData()
        resd.key = "img"
        resd.setDataType(DataText)
        resd.status = Once
        resd.setData(json.dumps(result_msg).encode("utf-8"))
        res.list = [resd]
        return res

    def wrapperFini(cls) -> int:
        return 0

    def wrapperError(cls, ret: int) -> str:
        if ret == 100:
            return "user error defined here"
        return ""

    def wrapperTestFunc(cls, data: [], respData: []):
        pass


if __name__ == '__main__':
    m = Wrapper()
    m.schema()
    m.run()



from .trainer_for_text_classification import TextClassificationTrainer
from .traniner_for_question_answer import QuestionAnswerTrainer
from .trainer_for_alpaca import AlpacaTrainer
from .trainer_for_llama_cn import Vicunatrainer
from .trainer_for_chinese_alpaca import ChineseAlpacaTrainer
from .trainer_for_baichuan import BaichuanTrainer
from .trainer_for_chatglm import ChatglmTrainer
from .trainer_for_ziya_llama_13b import Ziyallama13bTrainer



from typing import List
import torch
import os
from transformers import LlamaForCausalLM,AutoConfig,AutoModelForCausalLM,BitsAndBytesConfig
from ailab.atp_finetuner.model.model import AILabModel
from ailab.atp_finetuner.build import ModelRg
from ailab.atp_finetuner.constant import Task, Model

@ModelRg.register((Task.question_answering, Model.ziya_llama_13b))
class ZiyallamaModel(AILabModel):
    def __init__(self, model: any) -> None:
        super().__init__(model)

    def forward(self):
        pass
    
    @classmethod
    def build_model(cls, device_name:str, model_name:str, model_dir:str, **kwargs):
        model_name_or_dir = model_name if model_dir is None else model_dir

        # torch_dtype=torch.float16
        # config = AutoConfig.from_pretrained(model_name_or_dir)
        # 加载模型
        model = AutoModelForCausalLM.from_pretrained(
            model_name_or_dir,
            #device_map="auto",
            load_in_4bit=True,
            torch_dtype=torch.float16,
            trust_remote_code=True,
            quantization_config=BitsAndBytesConfig(
                load_in_4bit=True,
                bnb_4bit_compute_dtype=torch.float16,
                bnb_4bit_use_double_quant=True,
                bnb_4bit_quant_type="nf4",
                llm_int8_threshold=6.0,
                llm_int8_has_fp16_weight=False,
            ),
        )
        return cls(model)
    
    def get_inside_models(self, model_type:str):
        pass


#!/usr/bin/env python
import re
import subprocess
from ailab.log import logger

cmd = '''pip index versions pyatp-buildkit | grep -o  '(.*)' |cut -d  '('  -f2|cut -d  ')'  -f1|awk -F "." '{print $1"."$2"."$3+1}' '''


def get_latest_version(package_name):
    output = subprocess.run(["pip", "index", "versions", package_name],
                            capture_output=True)
    output = output.stdout.decode('utf-8')
    if output:
        output = list(filter(lambda x: len(x) > 0, output.split('\n')))
        latest_version = output[-1].split(':')[1].strip()
        return latest_version
    else:
        return None

def incre_version(version):
    regex = "\d+\.\d+\.\d+"
    if not re.match(regex, version):
        logger.info("not match version ")
        return  version
    o,tw,th = version.split(".")

    th = str(int(th) + 1)
    nv = '.'.join([o, tw,th])
    return nv


if __name__ == '__main__':
    result = get_latest_version('pyatp-buildkit')
    increment = incre_version(result)
    p = open("pyproject.toml",'r')
    fc = p.read()
    fc.replace("version= ")



# YOLOv5 🚀 by Ultralytics, GPL-3.0 license
"""
Train a YOLOv5 classifier model on a classification dataset

Usage - Single-GPU training:
    $ python classify/train.py --model yolov5s-cls.pt --data imagenette160 --epochs 5 --img 224

Usage - Multi-GPU DDP training:
    $ python -m torch.distributed.run --nproc_per_node 4 --master_port 2022 classify/train.py --model yolov5s-cls.pt --data imagenet --epochs 5 --img 224 --device 0,1,2,3

Datasets:           --data mnist, fashion-mnist, cifar10, cifar100, imagenette, imagewoof, imagenet, or 'path/to/data'
YOLOv5-cls models:  --model yolov5n-cls.pt, yolov5s-cls.pt, yolov5m-cls.pt, yolov5l-cls.pt, yolov5x-cls.pt
Torchvision models: --model resnet50, efficientnet_b0, etc. See https://pytorch.org/vision/stable/models.html
"""

import argparse
import os
import subprocess
import sys
import time
from copy import deepcopy
from datetime import datetime
from pathlib import Path

import torch
import torch.distributed as dist
import torch.hub as hub
import torch.optim.lr_scheduler as lr_scheduler
import torchvision
from torch.cuda import amp
from tqdm import tqdm

FILE = Path(__file__).resolve()
ROOT = FILE.parents[1]  # YOLOv5 root directory
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH
ROOT = Path(os.path.relpath(ROOT, Path.cwd()))  # relative

from classify import val as validate
from models.experimental import attempt_load
from models.yolo import ClassificationModel, DetectionModel
from utils.dataloaders import create_classification_dataloader
from utils.general import (DATASETS_DIR, LOGGER, TQDM_BAR_FORMAT, WorkingDirectory, check_git_info, check_git_status,
                           check_requirements, colorstr, download, increment_path, init_seeds, print_args, yaml_save)
from utils.loggers import GenericLogger
from utils.plots import imshow_cls
from utils.torch_utils import (ModelEMA, de_parallel, model_info, reshape_classifier_output, select_device, smart_DDP,
                               smart_optimizer, smartCrossEntropyLoss, torch_distributed_zero_first)

LOCAL_RANK = int(os.getenv('LOCAL_RANK', -1))  # https://pytorch.org/docs/stable/elastic/run.html
RANK = int(os.getenv('RANK', -1))
WORLD_SIZE = int(os.getenv('WORLD_SIZE', 1))
GIT_INFO = check_git_info()


def train(opt, device):
    init_seeds(opt.seed + 1 + RANK, deterministic=True)
    save_dir, data, bs, epochs, nw, imgsz, pretrained = \
        opt.save_dir, Path(opt.data), opt.batch_size, opt.epochs, min(os.cpu_count() - 1, opt.workers), \
        opt.imgsz, str(opt.pretrained).lower() == 'true'
    cuda = device.type != 'cpu'

    # Directories
    wdir = save_dir / 'weights'
    wdir.mkdir(parents=True, exist_ok=True)  # make dir
    last, best = wdir / 'last.pt', wdir / 'best.pt'

    # Save run settings
    yaml_save(save_dir / 'opt.yaml', vars(opt))

    # Logger
    logger = GenericLogger(opt=opt, console_logger=LOGGER) if RANK in {-1, 0} else None

    # Download Dataset
    with torch_distributed_zero_first(LOCAL_RANK), WorkingDirectory(ROOT):
        data_dir = data if data.is_dir() else (DATASETS_DIR / data)
        if not data_dir.is_dir():
            LOGGER.info(f'\nDataset not found ⚠️, missing path {data_dir}, attempting download...')
            t = time.time()
            if str(data) == 'imagenet':
                subprocess.run(['bash', str(ROOT / 'data/scripts/get_imagenet.sh')], shell=True, check=True)
            else:
                url = f'https://github.com/ultralytics/yolov5/releases/download/v1.0/{data}.zip'
                download(url, dir=data_dir.parent)
            s = f"Dataset download success ✅ ({time.time() - t:.1f}s), saved to {colorstr('bold', data_dir)}\n"
            LOGGER.info(s)

    # Dataloaders
    nc = len([x for x in (data_dir / 'train').glob('*') if x.is_dir()])  # number of classes
    trainloader = create_classification_dataloader(path=data_dir / 'train',
                                                   imgsz=imgsz,
                                                   batch_size=bs // WORLD_SIZE,
                                                   augment=True,
                                                   cache=opt.cache,
                                                   rank=LOCAL_RANK,
                                                   workers=nw)

    test_dir = data_dir / 'test' if (data_dir / 'test').exists() else data_dir / 'val'  # data/test or data/val
    if RANK in {-1, 0}:
        testloader = create_classification_dataloader(path=test_dir,
                                                      imgsz=imgsz,
                                                      batch_size=bs // WORLD_SIZE * 2,
                                                      augment=False,
                                                      cache=opt.cache,
                                                      rank=-1,
                                                      workers=nw)

    # Model
    with torch_distributed_zero_first(LOCAL_RANK), WorkingDirectory(ROOT):
        if Path(opt.model).is_file() or opt.model.endswith('.pt'):
            model = attempt_load(opt.model, device='cpu', fuse=False)
        elif opt.model in torchvision.models.__dict__:  # TorchVision models i.e. resnet50, efficientnet_b0
            model = torchvision.models.__dict__[opt.model](weights='IMAGENET1K_V1' if pretrained else None)
        else:
            m = hub.list('ultralytics/yolov5')  # + inference_wrapper.list('pytorch/vision')  # models
            raise ModuleNotFoundError(f'--model {opt.model} not found. Available models are: \n' + '\n'.join(m))
        if isinstance(model, DetectionModel):
            LOGGER.warning("WARNING ⚠️ pass YOLOv5 classifier model with '-cls' suffix, i.e. '--model yolov5s-cls.pt'")
            model = ClassificationModel(model=model, nc=nc, cutoff=opt.cutoff or 10)  # convert to classification model
        reshape_classifier_output(model, nc)  # update class count
    for m in model.modules():
        if not pretrained and hasattr(m, 'reset_parameters'):
            m.reset_parameters()
        if isinstance(m, torch.nn.Dropout) and opt.dropout is not None:
            m.p = opt.dropout  # set dropout
    for p in model.parameters():
        p.requires_grad = True  # for training
    model = model.to(device)

    # Info
    if RANK in {-1, 0}:
        model.names = trainloader.dataset.classes  # attach class names
        model.transforms = testloader.dataset.torch_transforms  # attach inference transforms
        model_info(model)
        if opt.verbose:
            LOGGER.info(model)
        images, labels = next(iter(trainloader))
        file = imshow_cls(images[:25], labels[:25], names=model.names, f=save_dir / 'train_images.jpg')
        logger.log_images(file, name='Train Examples')
        logger.log_graph(model, imgsz)  # log model

    # Optimizer
    optimizer = smart_optimizer(model, opt.optimizer, opt.lr0, momentum=0.9, decay=opt.decay)

    # Scheduler
    lrf = 0.01  # final lr (fraction of lr0)
    # lf = lambda x: ((1 + math.cos(x * math.pi / epochs)) / 2) * (1 - lrf) + lrf  # cosine
    lf = lambda x: (1 - x / epochs) * (1 - lrf) + lrf  # linear
    scheduler = lr_scheduler.LambdaLR(optimizer, lr_lambda=lf)
    # scheduler = lr_scheduler.OneCycleLR(optimizer, max_lr=lr0, total_steps=epochs, pct_start=0.1,
    #                                    final_div_factor=1 / 25 / lrf)

    # EMA
    ema = ModelEMA(model) if RANK in {-1, 0} else None

    # DDP mode
    if cuda and RANK != -1:
        model = smart_DDP(model)

    # Train
    t0 = time.time()
    criterion = smartCrossEntropyLoss(label_smoothing=opt.label_smoothing)  # loss function
    best_fitness = 0.0
    scaler = amp.GradScaler(enabled=cuda)
    val = test_dir.stem  # 'val' or 'test'
    LOGGER.info(f'Image sizes {imgsz} train, {imgsz} test\n'
                f'Using {nw * WORLD_SIZE} dataloader workers\n'
                f"Logging results to {colorstr('bold', save_dir)}\n"
                f'Starting {opt.model} training on {data} dataset with {nc} classes for {epochs} epochs...\n\n'
                f"{'Epoch':>10}{'GPU_mem':>10}{'train_loss':>12}{f'{val}_loss':>12}{'top1_acc':>12}{'top5_acc':>12}")
    for epoch in range(epochs):  # loop over the dataset multiple times
        tloss, vloss, fitness = 0.0, 0.0, 0.0  # train loss, val loss, fitness
        model.train()
        if RANK != -1:
            trainloader.sampler.set_epoch(epoch)
        pbar = enumerate(trainloader)
        if RANK in {-1, 0}:
            pbar = tqdm(enumerate(trainloader), total=len(trainloader), bar_format=TQDM_BAR_FORMAT)
        for i, (images, labels) in pbar:  # progress bar
            images, labels = images.to(device, non_blocking=True), labels.to(device)

            # Forward
            with amp.autocast(enabled=cuda):  # stability issues when enabled
                loss = criterion(model(images), labels)

            # Backward
            scaler.scale(loss).backward()

            # Optimize
            scaler.unscale_(optimizer)  # unscale gradients
            torch.nn.utils.clip_grad_norm_(model.parameters(), max_norm=10.0)  # clip gradients
            scaler.step(optimizer)
            scaler.update()
            optimizer.zero_grad()
            if ema:
                ema.update(model)

            if RANK in {-1, 0}:
                # Print
                tloss = (tloss * i + loss.item()) / (i + 1)  # update mean losses
                mem = '%.3gG' % (torch.cuda.memory_reserved() / 1E9 if torch.cuda.is_available() else 0)  # (GB)
                pbar.desc = f"{f'{epoch + 1}/{epochs}':>10}{mem:>10}{tloss:>12.3g}" + ' ' * 36

                # Test
                if i == len(pbar) - 1:  # last batch
                    top1, top5, vloss = validate.run(model=ema.ema,
                                                     dataloader=testloader,
                                                     criterion=criterion,
                                                     pbar=pbar)  # test accuracy, loss
                    fitness = top1  # define fitness as top1 accuracy

        # Scheduler
        scheduler.step()

        # Log metrics
        if RANK in {-1, 0}:
            # Best fitness
            if fitness > best_fitness:
                best_fitness = fitness

            # Log
            metrics = {
                'train/loss': tloss,
                f'{val}/loss': vloss,
                'metrics/accuracy_top1': top1,
                'metrics/accuracy_top5': top5,
                'lr/0': optimizer.param_groups[0]['lr']}  # learning rate
            logger.log_metrics(metrics, epoch)

            # Save model
            final_epoch = epoch + 1 == epochs
            if (not opt.nosave) or final_epoch:
                ckpt = {
                    'epoch': epoch,
                    'best_fitness': best_fitness,
                    'model': deepcopy(ema.ema).half(),  # deepcopy(de_parallel(model)).half(),
                    'ema': None,  # deepcopy(ema.ema).half(),
                    'updates': ema.updates,
                    'optimizer': None,  # optimizer.state_dict(),
                    'opt': vars(opt),
                    'git': GIT_INFO,  # {remote, branch, commit} if a git repo
                    'date': datetime.now().isoformat()}

                # Save last, best and delete
                torch.save(ckpt, last)
                if best_fitness == fitness:
                    torch.save(ckpt, best)
                del ckpt

    # Train complete
    if RANK in {-1, 0} and final_epoch:
        LOGGER.info(f'\nTraining complete ({(time.time() - t0) / 3600:.3f} hours)'
                    f"\nResults saved to {colorstr('bold', save_dir)}"
                    f'\nPredict:         python classify/predict.py --weights {best} --source im.jpg'
                    f'\nValidate:        python classify/val.py --weights {best} --data {data_dir}'
                    f'\nExport:          python export.py --weights {best} --include onnx'
                    f"\nPyTorch Hub:     model = torch.hub.load('ultralytics/yolov5', 'custom', '{best}')"
                    f'\nVisualize:       https://netron.app\n')

        # Plot examples
        images, labels = (x[:25] for x in next(iter(testloader)))  # first 25 images and labels
        pred = torch.max(ema.ema(images.to(device)), 1)[1]
        file = imshow_cls(images, labels, pred, de_parallel(model).names, verbose=False, f=save_dir / 'test_images.jpg')

        # Log results
        meta = {'epochs': epochs, 'top1_acc': best_fitness, 'date': datetime.now().isoformat()}
        logger.log_images(file, name='Test Examples (true-predicted)', epoch=epoch)
        logger.log_model(best, epochs, metadata=meta)


def parse_opt(known=False):
    parser = argparse.ArgumentParser()
    parser.add_argument('--model', type=str, default='yolov5s-cls.pt', help='initial weights path')
    parser.add_argument('--data', type=str, default='imagenette160', help='cifar10, cifar100, mnist, imagenet, ...')
    parser.add_argument('--epochs', type=int, default=10, help='total training epochs')
    parser.add_argument('--batch-size', type=int, default=64, help='total batch size for all GPUs')
    parser.add_argument('--imgsz', '--img', '--img-size', type=int, default=224, help='train, val image size (pixels)')
    parser.add_argument('--nosave', action='store_true', help='only save final checkpoint')
    parser.add_argument('--cache', type=str, nargs='?', const='ram', help='--cache images in "ram" (default) or "disk"')
    parser.add_argument('--device', default='', help='cuda device, i.e. 0 or 0,1,2,3 or cpu')
    parser.add_argument('--workers', type=int, default=8, help='max dataloader workers (per RANK in DDP mode)')
    parser.add_argument('--project', default=ROOT / 'runs/train-cls', help='save to project/name')
    parser.add_argument('--name', default='exp', help='save to project/name')
    parser.add_argument('--exist-ok', action='store_true', help='existing project/name ok, do not increment')
    parser.add_argument('--pretrained', nargs='?', const=True, default=True, help='start from i.e. --pretrained False')
    parser.add_argument('--optimizer', choices=['SGD', 'Adam', 'AdamW', 'RMSProp'], default='Adam', help='optimizer')
    parser.add_argument('--lr0', type=float, default=0.001, help='initial learning rate')
    parser.add_argument('--decay', type=float, default=5e-5, help='weight decay')
    parser.add_argument('--label-smoothing', type=float, default=0.1, help='Label smoothing epsilon')
    parser.add_argument('--cutoff', type=int, default=None, help='Model layer cutoff index for Classify() head')
    parser.add_argument('--dropout', type=float, default=None, help='Dropout (fraction)')
    parser.add_argument('--verbose', action='store_true', help='Verbose mode')
    parser.add_argument('--seed', type=int, default=0, help='Global training seed')
    parser.add_argument('--local_rank', type=int, default=-1, help='Automatic DDP Multi-GPU argument, do not modify')
    return parser.parse_known_args()[0] if known else parser.parse_args()


def main(opt):
    # Checks
    if RANK in {-1, 0}:
        print_args(vars(opt))
        check_git_status()
        check_requirements()

    # DDP mode
    device = select_device(opt.device, batch_size=opt.batch_size)
    if LOCAL_RANK != -1:
        assert opt.batch_size != -1, 'AutoBatch is coming soon for classification, please pass a valid --batch-size'
        assert opt.batch_size % WORLD_SIZE == 0, f'--batch-size {opt.batch_size} must be multiple of WORLD_SIZE'
        assert torch.cuda.device_count() > LOCAL_RANK, 'insufficient CUDA devices for DDP command'
        torch.cuda.set_device(LOCAL_RANK)
        device = torch.device('cuda', LOCAL_RANK)
        dist.init_process_group(backend='nccl' if dist.is_nccl_available() else 'gloo')

    # Parameters
    opt.save_dir = increment_path(Path(opt.project) / opt.name, exist_ok=opt.exist_ok)  # increment run

    # Train
    train(opt, device)


def run(**kwargs):
    # Usage: from yolov5 import classify; classify.train.run(data=mnist, imgsz=320, model='yolov5m')
    opt = parse_opt(True)
    for k, v in kwargs.items():
        setattr(opt, k, v)
    main(opt)
    return opt


if __name__ == '__main__':
    opt = parse_opt()
    main(opt)



#!/usr/bin/env python
# coding:utf-8
"""
@license: Apache License2
@file: wrapper.py
@time: 2023-03-02 13:17:14.585568
@project: chat_gpt_api_official
@project: ./
"""

import sys
import hashlib
try:
    from aiges_embed import ResponseData, Response, DataListNode, DataListCls  # c++
except:
    from aiges.dto import Response, ResponseData, DataListNode, DataListCls

from aiges.sdk import WrapperBase, \
    StringParamField, \
    ImageBodyField, \
    StringBodyField
from aiges.utils.log import log

########
# 请在此区域导入您的依赖库

# Todo
# for example: import numpy
import openai

########

'''
定义请求类:
 params:  params 开头的属性代表最终HTTP协议中的功能参数parameters部分，
          params Field支持 StringParamField，
          NumberParamField，BooleanParamField,IntegerParamField，每个字段均支持枚举
          params 属性多用于协议中的控制字段，请求body字段不属于params范畴

 input:    input字段多用与请求数据段，即body部分，当前支持 ImageBodyField, StringBodyField, 和AudioBodyField
'''


class UserRequest(object):
    # StringParamField多用于控制参数
    # 指明 enums, maxLength, required有助于自动根据要求配置协议schema
    params1 = StringParamField(key="p1", enums=["3", "eee"], value='3')
    params2 = StringParamField(key="p2", maxLength=44, required=True)
    params3 = StringParamField(key="p3", maxLength=44, required=False)

    # imagebodyfield 指明path，有助于本地调试wrapper.py
    input1 = ImageBodyField(key="data", path="test_data/test.png")
    input3 = ImageBodyField(key="data2", path="test_data/test.png")
    # stringbodyfiled 指明 value，用于本地调试时的测试值
    input2 = StringBodyField(key="switch", value="ctrl")


'''
定义响应类:
 accepts:  accepts代表响应中包含哪些字段, 以及数据类型

 input:    input字段多用与请求数据段，即body部分，当前支持 ImageBodyField, StringBodyField, 和AudioBodyField
'''


class UserResponse(object):
    # 此类定义响应返回数据段，请务必指明对应key
    # 支持 ImageBodyField， AudioBodyField,  StringBodyField
    # 如果响应是json， 请使用StringBodyField
    accept1 = StringBodyField(key="boxes")
    accept2 = StringBodyField(key="boxes2")


'''
用户实现， 名称必须为Wrapper, 必须继承SDK中的 WrapperBase类
'''


class Wrapper(WrapperBase):
    serviceId = "chatgpt"
    version = "v1"
    requestCls = UserRequest()
    responseCls = UserResponse()

    '''
    服务初始化
    @param config:
        插件初始化需要的一些配置，字典类型
        key: 配置名
        value: 配置的值
    @return
        ret: 错误码。无错误时返回0
    '''

    def wrapperInit(cls, config: {}) -> int:
        log.info(config)
        log.info("Initializing ...")
        # doc is here https://platform.openai.com/docs/guides/chat/chat-vs-completions?utm_medium=email&_hsmi=248334739&utm_content=248334739&utm_source=hs_email
        chat_completion = openai.ChatCompletion.create(
            model="gpt-3.5-turbo",
            messages=[
                # system message first, it helps set the behavior of the assistant
                {"role": "system", "content": "You are a helpful assistant."},
                # I am the user, and this is my prompt
                {"role": "user", "content": "What's the best star wars movie?"},
                # we can also add the previous conversation
                # {"role": "assistant", "content": "Episode III."},
            ],
        )
        return 0

    '''
    非会话模式计算接口,对应oneShot请求,可能存在并发调用
    @param params 功能参数
    @param  reqData     请求数据实体字段 DataListCls,可通过 aiges.dto.DataListCls查看
    @return 
        响应必须返回 Response类，非Response类将会引起未知错误
    '''

    def wrapperOnceExec(cls, params: {}, reqData: DataListCls) -> Response:
        log.info("got reqdata , %s" % reqData.list)
        #        print(type(reqData.list[0].data))
        #        print(type(reqData.list[0].data))
        #        print(reqData.list[0].len)
        for req in reqData.list:
            log.info("reqData key: %s , size is %d" % (req.key, len(req.data)))
        log.warning("reqData bytes md5sum is %s" % hashlib.md5(reqData.list[0].data).hexdigest())
        log.info("I am infer logic...please inplement")
        log.info("Testing reqData get: ")
        rg = reqData.get("data")
        log.info("get key: %s" % rg.key)
        log.info("get key: %d" % len(rg.data))

        # test not reqdata
        k = "dd"
        n = reqData.get(k)
        if not n:
            log.error("reqData not has this key %s" % k)

        log.warning("reqData bytes md5sum is %s" % hashlib.md5(reqData.list[0].data).hexdigest())
        log.info("I am infer logic...please inplement")
        r = Response()
        # 错误处理
        # return r.response_err(100)
        l = ResponseData()
        l.key = "ccc"
        l.status = 1
        d = open("test_data/test.png", "rb").read()
        l.len = len(d)
        l.data = d
        l.type = 0
        r.list = [l, l, l]
        return r

    '''
    服务逆初始化

    @return
        ret:错误码。无错误码时返回0
    '''

    def wrapperFini(cls) -> int:
        return 0

    '''
    非会话模式计算接口,对应oneShot请求,可能存在并发调用
    @param ret wrapperOnceExec返回的response中的error_code 将会被自动传入本函数并通过http响应返回给最终用户
    @return 
        str 错误提示会返回在接口响应中
    '''

    def wrapperError(cls, ret: int) -> str:
        if ret == 100:
            return "user error defined here"
        return ""

    '''
        此函数保留测试用，不可删除
    '''

    def wrapperTestFunc(cls, data: [], respData: []):
        r = Response()
        l = ResponseData()
        l.key = "ccc"
        l.status = 1
        d = open("pybind11/docs/pybind11-logo.png", "rb").read()
        l.len = len(d)
        l.data = d
        r.list = [l, l, l]
        return r


if __name__ == '__main__':
    m = Wrapper()
    # m.schema()
    m.run()


import os
from typing import Callable
import torch
from transformers import TrainingArguments, Trainer
from peft import get_peft_model_state_dict, set_peft_model_state_dict
from ailab.atp_dataset.dataset import AILabDataset
from ailab.atp_finetuner.trainer import AILabTrainer 
from ailab.atp_finetuner.model import AILabModel
from ailab.atp_finetuner.datacollator import AILabDataCollator
from ailab.atp_finetuner.metric import AILabMetric
from ailab.atp_finetuner.preprossor import AILabPreprocessor
from ailab.atp_finetuner.build import TrainerRg
from ailab.atp_finetuner.constant import Task, Model
from ailab.utils.callbacks import TrainProgress
from ailab.log import logger

@TrainerRg.register((Task.question_answering_cn, Model.vicuna))
@TrainerRg.register((Task.question_answering_cn, Model.open_llama))
class Vicunatrainer(AILabTrainer):
    def __init__(self):
        super().__init__()

    def preprocess(self, dataset:AILabDataset, model:AILabModel, preprocessor: AILabPreprocessor, \
                      data_collator:AILabDataCollator, metric:AILabMetric, train_progress:Callable, **kwargs):
        train_args = kwargs['train_args']
        output_dir = train_args.get('output_dir', 'my_model')
        learning_rate = train_args.get('learning_rate', 1e-5)
        num_train_epochs = train_args.get('num_train_epochs', 2)
        evaluation_strategy = train_args.get('evaluation_strategy', "epoch")
        save_strategy = train_args.get('save_strategy', "epoch")
        per_device_train_batch_size = train_args.get('per_device_train_batch_size', 16)
        gradient_accumulation_steps = train_args.get('gradient_accumulation_steps', 4)
        per_device_eval_batch_size = train_args.get('per_device_eval_batch_size', 16)
        weight_decay = train_args.get('weight_decay', 0.01)
        logging_steps = train_args.get('logging_steps', 10)
        warmup_steps = train_args.get('warmup_steps', 100)
        fp16 = train_args.get('fp16', False)
        bf16 = train_args.get('bf16', True)
        optim = train_args.get('optim', "adamw_torch")
        eval_steps = train_args.get('eval_steps', 200)
        save_steps = train_args.get('save_steps', 200)
        max_steps = train_args.get('max_steps', 5000)
        resume_from_checkpoint = train_args.get('resume_from_checkpoint', False)
        
        deepspeed_dir = os.path.dirname(os.path.abspath(__file__))
        deepspeed_dir = os.path.join(deepspeed_dir,"ds_zero2_no_offload.json")
        training_args = TrainingArguments(
            output_dir=output_dir,
            evaluation_strategy=evaluation_strategy,
            save_strategy=save_strategy,
            learning_rate=learning_rate,
            per_device_train_batch_size=per_device_train_batch_size,
            per_device_eval_batch_size=per_device_eval_batch_size,
            num_train_epochs=num_train_epochs,
            weight_decay=weight_decay,
            gradient_accumulation_steps = gradient_accumulation_steps,
            logging_steps = logging_steps,
            warmup_steps = warmup_steps,
            fp16 = fp16,
            bf16 = bf16,
            optim = optim,
            eval_steps = eval_steps,
            save_steps = save_steps,
            save_total_limit=3,
            load_best_model_at_end=False,
            report_to=[],
            ignore_data_skip=False,
            push_to_hub=False,
            ddp_find_unused_parameters=False,
            ddp_timeout=30000,
            deepspeed=deepspeed_dir,
        )

        logger.info(f'Vicunatrainer training_args {training_args}')

        trainer = Trainer(
            model=model.model_ins,
            args=training_args,
            train_dataset=dataset.to_hf_dataset()["train"],
            eval_dataset=dataset.to_hf_dataset()["test"],
            data_collator=data_collator.datacollator_ins,
            callbacks=[TrainProgress(train_progress)],
        )
        self.trainer = trainer
        self.model = model.model_ins
        self.output_dir = output_dir
        self.resume_from_checkpoint = resume_from_checkpoint

    def train(self):
        model = self.model
        resume_from_checkpoint = self.resume_from_checkpoint

        from transformers.trainer_utils import get_last_checkpoint
        if resume_from_checkpoint:
            resume_from_checkpoint = get_last_checkpoint(self.output_dir)
            if resume_from_checkpoint is None:
                resume_from_checkpoint = False
            else:
                checkpoint_name = os.path.join(resume_from_checkpoint, "pytorch_model.bin")
                if not os.path.exists(checkpoint_name):
                    checkpoint_name = os.path.join(resume_from_checkpoint, "adapter_model.bin")
                if os.path.exists(checkpoint_name):
                    logger.info(f"Restarting from {checkpoint_name}")
                    adapters_weights = torch.load(checkpoint_name)
                    set_peft_model_state_dict(model, adapters_weights)
                else:
                    resume_from_checkpoint = False

        model.config.use_cache = False
        old_state_dict = model.state_dict
        model.state_dict = (
            lambda self, *_, **__: get_peft_model_state_dict(
                self, old_state_dict()
            )
        ).__get__(model, type(model))
        model = torch.compile(model)

        self.trainer.train(resume_from_checkpoint=resume_from_checkpoint)
        model.save_pretrained(self.output_dir)

    def postprocess(self):
        self.trainer.evaluate()






import json
import logging


class JsonFileHandler(logging.FileHandler):
    def __init__(self, filename, mode="a", encoding=None, delay=False):
        super().__init__(filename, mode, encoding, delay)

    def emit(self, record):
        json_data = json.loads(self.format(record))
        with open(self.baseFilename, "w", encoding="utf-8") as f:
            json.dump(json_data, f, ensure_ascii=False, indent=4)


import logging


class JsonFormatter(logging.Formatter):
    def format(self, record):
        return record.msg



import os.path
import pathlib
import shutil
from typing import Any
import logging
from jinja2 import Environment
from plumbum import cli
from plumbum.cmd import rm  # type: ignore
import subprocess
from pyatp_buildkit.config import *
from ailab.inference_wrapper.huggingface import transformers
import datetime

log = logging.getLogger()
pwd = os.path.abspath(__file__)
dockerfile_dir = ''
import subprocess


class Manager(cli.Application):
    """aialb CI Manager"""

    PROGNAME: str = "build.py"
    VERSION: str = "0.0.1"

    manifest = {}
    ci = None

    def main(self):
        if not self.nested_command:  # will be ``None`` if no sub-command follows
            log.fatal("No subcommand given!")
            print()
            self.help()
            return 1
        elif len(self.nested_command[1]) < 2 and any(
                "generate" in arg for arg in self.nested_command[1]
        ):
            log.error(
                "Subcommand 'generate' missing  required arguments! use 'generate --help'"
            )
            return 1


@Manager.subcommand("generate")  # type: ignore
class ManagerGenerate(Manager):
    DESCRIPTION = "Generate Dockerfiles from templates."

    parent: Manager

    vars = {}

    matrix = []
    template_env: Any = Environment(
        extensions=["jinja2.ext.do", "jinja2.ext.loopcontrols"],
        trim_blocks=True,
        lstrip_blocks=True,
    )
    cache: Any = cli.Flag(
        ["-c", "--cache"], default=True, help="Enable cache mode"
    )
    template: Any
    action: Any = cli.SwitchAttr(
        "--action",
        str,
        group="Targeted",

        excludes=["--all", ],
        default='build',
    )

    build_name: Any = cli.SwitchAttr(
        "--build_name",
        str,
        group="Targeted",

        excludes=["--all", ],
        default='test_aiservice',
    )
    dockerfile: Any = cli.SwitchAttr(
        "--dockerfile",
        str,
        group="Targeted",
        excludes=["--all", ],
        default='',
    )
    generate_all: Any = cli.Flag(
        ["--all"],
        help="Generate all of the templates.",
    )

    distro: Any = cli.SwitchAttr(
        "--model",
        str,
        group="Targeted",
        excludes=["--all", ],
        help="The distro to use.",
        default=None,
    )
    pretrained_model: Any = cli.SwitchAttr(
        "--pretrained_model",
        str,
        group="Targeted",
        excludes=["--all"],
        help="The distro version",
    )

    model_name: Any = cli.SwitchAttr(
        "--model_name",
        str,
        group="Targeted",
        excludes=["--all"],
        help="The distro version",
    )

    model_path: Any = cli.SwitchAttr(
        "--model_path",
        str,
        group="Targeted",
        excludes=["--all"],
        help="The distro version",
        default='',
    )
    is_transformers: Any = cli.Flag(
        ["--is_transformers"],
        help="If Using Github Actions",
        default=True,
    )

    inference_task: Any = cli.SwitchAttr(
        "--inference_task",
        str,
        excludes=["--all"],
        group="Targeted",
        help="The cuda version to use. Example: '11.2'",
        default=None,
    )

    inference_script_path: Any = cli.SwitchAttr(
        "--inference_script_path",
        str,
        group="Targeted",
        excludes=["--all", ],
        help="inference_script_path",
        default='',
    )

    project_id: Any = cli.SwitchAttr(
        "--project_id",
        str,
        group="Targeted",
        excludes=["--all", ],
        help="git tag ",
        default=None,
    )
    base_image: Any = cli.SwitchAttr(
        "--base_image",
        str,
        group="Targeted",
        excludes=["--all", ],
        help="base_image tag ",
        default="artifacts.iflytek.com/docker-private/atp/atp_general_image:v1.0.9",
    )
    docker_repo: Any = cli.SwitchAttr(
        "--docker_repo",
        str,
        group="Targeted",
        excludes=["--all", ],
        help="",
        default="artifacts.iflytek.com/docker-private/atp/train",
    )
    maintainer: Any = cli.SwitchAttr(
        "--maintainer",
        str,
        group="Targeted",
        excludes=["--all", ],
        help="image entrypoint ",
        default="ybyang7@iflytek.com",
    )
    image_tag: Any = cli.SwitchAttr(
        "--tag",
        str,
        group="Targeted",
        excludes=["--all", ],
        default="latest",
    )
    entrypoint: Any = cli.SwitchAttr(
        "--entrypoint",
        str,
        group="Targeted",
        excludes=["--all", ],
        help="image entrypoint ",
        default=None,
    )

    cmd: Any = cli.SwitchAttr(
        "--cmd",
        str,
        group="Targeted",
        excludes=["--all", ],
        help="iamge cmd ",
        default=None,
    )

    extra_pip_packages: Any = cli.SwitchAttr(
        '--extra_pip_packages',
        str,
        group="Targeted",
        excludes=["--all", ],
        default="aiges pyatp",
    )
    auto_run: Any = cli.Flag(
        ["--auto_run"],
        help="If Using Github Actions",
        default=True,
    )

    # extracts arbitrary keys and inserts them into the templating context
    def extract_keys(self, val, arch=None):
        pass

    # For cudnn templates, we need a custom template context
    def output_cudnn_template(self, cudnn_version_name, input_template, output_path):
        pass

    def prepare_context(self):

        # The templating context. This data structure is used to fill the templates.
        self.vars = {
            "registry": self.get_regsitry(),
            "tag": self.generate_matrix_tags(),
        }

    def prepare_packages_txt(self) -> str:
        packages_txt = ""
        return packages_txt

    def prepare_model(self):
        model = self.model_name
        self.vars['model'] = model
        model_path = self.model_path

        if os.path.exists(model_path):
            self.vars['model_path'] = model_path

    def check_inference_path(self, p):
        # todo 校验真实p地址
        if not os.path.exists(p):
            log.error("not exists this inference dir %s" % p)
            return False
        if not os.path.isdir(p):
            log.error("not a  dir")
            return False

        else:
            files = os.listdir(p)
            if 'wrapper' not in files:
                log.error("please specify a dir which contains a wrapper.py")
                return False
            else:
                _path = os.path.join(p, 'wrapper')
                if not os.path.isdir(_path):
                    return False
                files2 = os.listdir(_path)
                if "wrapper.py" not in files2:
                    return False

        return True

    def prepare_inference_task_and_module(self):
        python_path = ''
        global dockerfile_dir
        # todo 拷贝推理文件以及目录
        if self.inference_script_path:
            valid = self.check_inference_path(self.inference_script_path)
            if valid:
                log.info("copy files to workspace")
                files = os.listdir(self.inference_script_path)
                target_dir = os.path.join(dockerfile_dir, 'wrapper')
                #                if not os.path.exists(target_dir):
                #                    os.makedirs(target_dir)
                for f in files:
                    if os.path.isdir(os.path.join(self.inference_script_path, f)):
                        shutil.copytree(os.path.join(self.inference_script_path, f), target_dir, dirs_exist_ok=True)
                    else:
                        shutil.copy(os.path.join(self.inference_script_path, f), os.path.join(dockerfile_dir, f))

                if 'requirements.txt' in files:
                    log.info("finding. requirements.txt")
                    self.vars['requirements_txt'] = "requirements.txt"
                if 'packages.txt' in files:
                    log.info("finding. packages.txt")
                    self.vars['packages_txt'] = "packages.txt"

            elif self.pretrained_model:
                # 使用pretrained_model 预制
                if not self.pretrained_model in SUPPORTED_TRAIN_TASKS:
                    raise ModuleNotFoundError("Not support this pretrained_model")
                log.info("copy pretrained innternal files to workspace")
                self.inference_script_path = SUPPORTED_TRAIN_TASKS_WRAPPER_MAP[self.pretrained_model]
                files = os.listdir(self.inference_script_path)

                target_dir = os.path.join(dockerfile_dir, 'wrapper')
                #                if not os.path.exists(target_dir):
                #                    os.makedirs(target_dir)
                for f in files:
                    if os.path.isdir(os.path.join(self.inference_script_path, f)):
                        shutil.copytree(os.path.join(self.inference_script_path, f), target_dir)
                    else:
                        shutil.copy(os.path.join(self.inference_script_path, f), os.path.join(dockerfile_dir, f))

                if 'requirements.txt' in files:
                    log.info("finding. requirements.txt")
                    self.vars['requirements_txt'] = "requirements.txt"
                if 'packages.txt' in files:
                    log.info("finding. packages.txt")
                    self.vars['packages_txt'] = "packages.txt"

            else:
                raise FileNotFoundError(self.inference_script_path)
        # transformer 内置
        elif self.is_transformers and self.inference_task:
            valid, m_path_dict = self.check_inference_task(self.inference_task)
            if valid:
                self.vars['inference_wrapper_path'] = m_path_dict[self.inference_task]
                log.info("is transformer.. using internal module: %s" % m_path_dict[self.inference_task])

            else:
                log.info("not valid %s" % self.inference_task)

                self.vars['inference_wrapper_path'] = "."
        else:
            self.vars['inference_wrapper_path'] = python_path

    def check_inference_task(self, task):
        transformers_dir = os.path.dirname(transformers.__file__)
        SUPPORTED_TASK = {}
        path_dict = {}
        fs = os.listdir(transformers_dir)
        for d in fs:
            if os.path.isdir(os.path.join(transformers_dir, d)):
                if d == "__pycache__":
                    continue
                if d not in SUPPORTED_TASK:
                    SUPPORTED_TASK[d] = []
                tasks = os.listdir(os.path.join(transformers_dir, d))
                for task in tasks:
                    p = os.path.join(transformers_dir, d, task)
                    if os.path.isdir(p):
                        SUPPORTED_TASK[d].append(task)
                        path_dict[task] = p
        valid = False
        for k in SUPPORTED_TASK.values():
            if task in k:
                valid = True
        return valid, path_dict

    def prepare_requirements_txt(self) -> str:
        requirements_txt = ""
        return requirements_txt

    def generate_dockerfile(self):
        if os.path.exists(TEMP_GEN_DIR):
            shutil.rmtree(TEMP_GEN_DIR)

        if not os.path.exists(TEMP_GEN_DIR):
            os.makedirs(TEMP_GEN_DIR)
        dockerfile_content = ''
        if os.path.exists(self.dockerfile):
            dockerfile_content = open(self.dockerfile).read()

        self.vars["base_image"] = self.base_image
        self.vars['maintainer'] = self.maintainer
        self.vars['pip_source'] = 'https://pypi.mirrors.ustc.edu.cn/simple/'

        if dockerfile_content:
            self.vars['dockerfile'] = dockerfile_content
        global dockerfile_dir
        dockerfile_dir = os.path.join(TEMP_GEN_DIR, 'run')
        if not os.path.exists(dockerfile_dir):
            os.makedirs(dockerfile_dir)

        self.prepare_inference_task_and_module()
        # extra pip packages
        if self.extra_pip_packages:
            self.vars['extra_pip_packages'] = self.extra_pip_packages

        # for render
        render_vars = {
            "vars": self.vars
        }
        st = self.render(render_vars)
        log.info("Generated Dockerfile %s " % st)
        with open(os.path.join(dockerfile_dir, Dockerfile), 'w') as dockerfile:
            dockerfile.write(st)
            dockerfile.close()
            log.info("write %s success" % os.path.abspath(os.path.join(dockerfile_dir, Dockerfile)))

        if self.auto_run:
            self.buildx()

    def mock_buildx(self):
        global dockerfile_dir
        log.info("buildx dockerfile in %s" % dockerfile_dir)
        log.info("building: ")
        with open(os.path.abspath(os.path.join(dockerfile_dir, Dockerfile)), 'r') as f:
            log.info(f.read())
        f.close()
        if not self.image_tag:
            image_tag = "latest"
        cache = " --no-cache "
        if self.cache:
            cache = ""

        cmd = f"docker buildx build . -t {self.docker_repo}/{self.build_name}:{self.image_tag} {cache} --push"
        log.info(cmd)

    def buildx(self):
        # 1. check buildx command
        global dockerfile_dir
        mock = True
        docker_bin = "/usr/local/bin/docker"
        if os.path.exists(docker_bin):
            mock = False

        if mock:
            self.mock_buildx()
            return
        pwd = os.getcwd()
        cache = " --no-cache "
        if self.cache:
            cache = ""
        cmd = f"docker buildx build . -t {self.docker_repo}/{self.build_name}:{self.image_tag} {cache} --push"

        log.info(cmd)
        os.chdir(dockerfile_dir)
        subprocess.call(cmd, shell=True)
        os.chdir(pwd)

    def render(self, render_vars):
        s = self.template.render(**render_vars)
        return s

    def get_regsitry(self):
        if self.use_github:
            return ECR_REPO
        return INNER_REPO

    def _load_template(self):
        tpl = os.path.join(os.path.dirname(pwd), "templates/Dockerfile.j2")
        if not os.path.exists(tpl):
            raise FileNotFoundError("not found %s" % tpl)
        log.info("load success j2 file.")
        self.template = self.template_env.from_string(open(tpl, "r").read())

    def _load_release_note(self):
        tpl = "./docker/templates/release-note/Note.md.j2"
        if not os.path.exists(tpl):
            raise FileNotFoundError("not found %s" % tpl)
        log.info("load success Note.md j2 file.")
        self.release_note = self.template_env.from_string(open(tpl, "r").read())

    def find_latest_gz(self):
        # 列出目录下所有的文件
        testdir = "."
        list = os.listdir(".")
        gzs = [x for x in list if x.startswith("pyatp") and x.endswith(".tar.gz")]
        # 对文件修改时间进行升序排列
        gzs.sort(key=lambda fn: os.path.getmtime(os.path.join(testdir, fn)))
        # 获取最新修改时间的文件
        filetime = datetime.datetime.fromtimestamp(os.path.getmtime(os.path.join(testdir, gzs[-1])))
        # 获取文件所在目录
        filepath = os.path.join(testdir, gzs[-1])
        print("find latest %s" % filepath)
        return filepath

    def upgrade_ailab_dir(self):
        print(".. Upgrading bulletin ailab build sdk!")

        cmd = " pip download  pyatp --no-deps --no-binary=:all:"
        subprocess.call(cmd, shell=True)

        latest_gz = self.find_latest_gz()

        cmds = [
            "pip config set global.index-url https://repo.model.xfyun.cn/api/packages/administrator/pypi/simple  &&  pip config set global.extra-index-url https://pypi.mirrors.ustc.edu.cn/simple/",
            'pip install  cchardet ',
        ]
        if latest_gz and os.path.exists(latest_gz):
            print("upgrading ailab dir: %s" % latest_gz)
            mkdir = "mkdir -p /home/ailab"
            unpress = "tar zxvf %s --strip-components 2 -C /home/ailab" % latest_gz
            cmds.append(mkdir)
            cmds.append(unpress)

        for cm in cmds:
            subprocess.call(cm, shell=True)

    def targeted(self):
        try:
            self.upgrade_ailab_dir()
        except Exception as e:
            print(e)

        if self.action == "build":
            log.info("building generating")
            self._load_template()
            self.generate_dockerfile()

        elif self.action == "release":
            log.info("releasing generating...")
        else:
            log.error("wrong action %s" % self.action)

    def release(self):
        pass

    def main(self):
        self.targeted()
        log.info("Done")


if __name__ == "__main__":
    Manager.run()



import os
import time
import numpy as np

import pandas as pd
from tqdm import tqdm
from ailab.atp_evaluation.benchmarks.base import AILabBenchmark
from ailab.atp_evaluation.build import BenchmarkRg
from ailab.atp_evaluation.constant import BenchMarkType
from ailab.atp_evaluation.models.base import AILabModel
from ailab.log import logger

subcategories = {
    "abstract_algebra": ["math"],
    "anatomy": ["health"],
    "astronomy": ["physics"],
    "business_ethics": ["business"],
    "clinical_knowledge": ["health"],
    "college_biology": ["biology"],
    "college_chemistry": ["chemistry"],
    "college_computer_science": ["computer science"],
    "college_mathematics": ["math"],
    "college_medicine": ["health"],
    "college_physics": ["physics"],
    "computer_security": ["computer science"],
    "conceptual_physics": ["physics"],
    "econometrics": ["economics"],
    "electrical_engineering": ["engineering"],
    "elementary_mathematics": ["math"],
    "formal_logic": ["philosophy"],
    "global_facts": ["other"],
    "high_school_biology": ["biology"],
    "high_school_chemistry": ["chemistry"],
    "high_school_computer_science": ["computer science"],
    "high_school_european_history": ["history"],
    "high_school_geography": ["geography"],
    "high_school_government_and_politics": ["politics"],
    "high_school_macroeconomics": ["economics"],
    "high_school_mathematics": ["math"],
    "high_school_microeconomics": ["economics"],
    "high_school_physics": ["physics"],
    "high_school_psychology": ["psychology"],
    "high_school_statistics": ["math"],
    "high_school_us_history": ["history"],
    "high_school_world_history": ["history"],
    "human_aging": ["health"],
    "human_sexuality": ["culture"],
    "international_law": ["law"],
    "jurisprudence": ["law"],
    "logical_fallacies": ["philosophy"],
    "machine_learning": ["computer science"],
    "management": ["business"],
    "marketing": ["business"],
    "medical_genetics": ["health"],
    "miscellaneous": ["other"],
    "moral_disputes": ["philosophy"],
    "moral_scenarios": ["philosophy"],
    "nutrition": ["health"],
    "philosophy": ["philosophy"],
    "prehistory": ["history"],
    "professional_accounting": ["other"],
    "professional_law": ["law"],
    "professional_medicine": ["health"],
    "professional_psychology": ["psychology"],
    "public_relations": ["politics"],
    "security_studies": ["politics"],
    "sociology": ["culture"],
    "us_foreign_policy": ["politics"],
    "virology": ["health"],
    "world_religions": ["philosophy"],
}
categories = {
    "STEM": ["physics", "chemistry", "biology", "computer science", "math", "engineering"],
    "humanities": ["history", "philosophy", "law"],
    "social sciences": ["politics", "culture", "economics", "geography", "psychology"],
    "other (business, health, misc.)": ["other", "business", "health"],
}

@BenchmarkRg.register(BenchMarkType.mmlu)
class MMLUBenchmark(AILabBenchmark):
    def __init__(self, model: AILabModel, dataset_dir: str, ntrain: int, output_dir: str, **kwargs) -> None:
        super().__init__()
        outpath = os.path.join(output_dir, r"logs_mmlu")
        if not os.path.exists(outpath):
            os.mkdir(outpath)
        run_date = time.strftime('%Y-%m-%d_%H-%M-%S', time.localtime(time.time()))
        logger.info(model.model_name)
        result_dir_prefix = model.model_name.replace("/", "_")
        self._result_dir = os.path.join(outpath, f"{result_dir_prefix}_{run_date}")
        os.mkdir(self._result_dir)
        if not os.path.exists(dataset_dir):
            raise TypeError(f'{dataset_dir} is not exist')
        self._dataset_dir = dataset_dir
        self._kshot = ntrain
        self._model = model
        self._result_file = f'mmlu_{result_dir_prefix}_{ntrain}_shot.txt'
    
    def evaluate(self):
        subjects = sorted(
            [
                f.split("_test.csv")[0]
                for f in os.listdir(os.path.join(self._dataset_dir, "test"))
                if "_test.csv" in f
            ]
        )
        logger.info(subjects)
        all_cors = []
        subcat_cors = {
            subcat: [] for subcat_lists in subcategories.values() for subcat in subcat_lists
        }
        cat_cors = {cat: [] for cat in categories}
        samples = 0
        for sub_name in subjects:
            dev_df = pd.read_csv(
                os.path.join(self._dataset_dir, "dev", sub_name + "_dev.csv"), header=None
            )[: self._kshot]
            test_df = pd.read_csv(
                os.path.join(self._dataset_dir, "test", sub_name + "_test.csv"), header=None
            )
            cors = self.__eval_subject(sub_name, test_df, dev_df, self._kshot)
            samples += test_df.shape[0]
            acc = np.mean(cors)
            with open(os.path.join(self._result_dir, self._result_file), "a") as f:
                f.write("Average accuracy {:.3f} - {}({})\n".format(acc, sub_name, test_df.shape[0]))
            subcats = subcategories[sub_name]
            for subcat in subcats:
                subcat_cors[subcat].append(cors)
                for key in categories.keys():
                    if subcat in categories[key]:
                        cat_cors[key].append(cors)
            all_cors.append(cors)

        with open(os.path.join(self._result_dir, self._result_file), "a") as f:
            for subcat in subcat_cors:
                subcat_acc = np.mean(np.concatenate(subcat_cors[subcat]))
                f.write("Average accuracy {:.3f} - {}\n".format(subcat_acc, subcat))

            for cat in cat_cors:
                cat_acc = np.mean(np.concatenate(cat_cors[cat]))
                f.write("Average accuracy {:.3f} - {}\n".format(cat_acc, cat))

            weighted_acc = np.mean(np.concatenate(all_cors))
            f.write("Average accuracy: {:.3f}({})\n".format(weighted_acc, samples))

    def __eval_subject(self, subject_name, test_df, dev_df=None, kshot=-1):
        results = {}
        cors = []
        train_prompt = self.__gen_few_shot_prompt(dev_df, subject_name, kshot)
        for i in tqdm(range(test_df.shape[0])):
            question = self.__format_example(test_df, i, include_answer=False)
            prompt = train_prompt + question
            # logger.info(prompt)
            ans = self._model.get_answer_of_multiple_choices_question(prompt)
            label = test_df.iloc[i, test_df.shape[1] - 1]
            # logger.info(ans)
            cor = ans == label
            # logger.info(cor)
            cors.append(cor)
        cors = np.array(cors)
        return cors

    def __format_example(self, df, idx, include_answer=True):
        prompt = df.iloc[idx][0]
        k = df.shape[1] - 2
        for j in range(k):
            prompt += "\n{}. {}".format(self._choices[j], df.iloc[idx, j + 1])
        prompt += "\nAnswer:"
        if include_answer:
            prompt += " {}\n\n".format(df.iloc[idx, k + 1])
        return prompt

    def __gen_few_shot_prompt(self, train_df, subject, k=-1):
        prompt = "The following is multiple choice question about {}.\n\n".format(
            self._format_subject(subject)
        )
        if k == -1:
            k = train_df.shape[0]
        if k > 0:
            prompt = "The following are multiple choice questions (with answers) about {}.\n\n".format(
                self._format_subject(subject)
            )
        for i in range(k):
            prompt += self.__format_example(train_df, i)
        return prompt


import evaluate
import numpy as np
from ailab.atp_finetuner.metric import AILabMetric
from ailab.atp_finetuner.build import MetricRg
from ailab.atp_finetuner.constant import Task, Model

@MetricRg.register((Task.text_classification, Model.distilbert_base_uncased))
class AccuryMetric(AILabMetric):
    def __init__(self) -> None:
        super().__init__()
    
    @staticmethod
    def evalute(eval_pred) :
        accuracy = evaluate.load("accuracy")
        predictions, labels = eval_pred
        predictions = np.argmax(predictions, axis=1)
        return accuracy.compute(predictions=predictions, references=labels)
    


from typing import Callable, Optional
from ailab.atp_dataset.dataset import AILabDataset
from ailab.atp_finetuner.constant import Framework, Task, Model
from ailab.atp_finetuner.preprossor import AILabPreprocessor
from ailab.atp_finetuner.model import AILabModel
from ailab.atp_finetuner.datacollator import AILabDataCollator
from ailab.atp_finetuner.metric import AILabMetric
from ailab.atp_finetuner.trainer import AILabTrainer
from ailab.atp_finetuner.accelerator import AILabAccelerator

class AILabFinetuner:
    def __init__(self, task: Task, framework: Framework.Pytorch, 
                dataset: AILabDataset, model_name: Model, 
                train_progress:Callable, 
                model_dir: Optional[str] = None,
                pc_dir: Optional[str] = None, 
                **args):
        accelerator = AILabAccelerator.from_project_config(**args)
        preprocessor = AILabPreprocessor.from_pretrained(task, model_name, dataset, pc_dir)
        preprocessor.accelerator = accelerator
        tokenized_dataset = preprocessor.process_data()

        model = AILabModel.from_pretrained('cuda', task, model_name, model_dir, **args)
        model.accelerator = accelerator
        model.forward()

        data_collator = AILabDataCollator.from_task_model(task, model_name, framework, preprocessor, model)
        metrics = AILabMetric.from_task_model(task, model_name)
        trainer = AILabTrainer.from_task_model(task, model_name, tokenized_dataset, model, preprocessor, data_collator,
                                               metrics, train_progress, **args)
        trainer.accelerator = accelerator

        self._trainer = trainer

    def finetuner(self):
        self._trainer.train()



import openai
from langchain.chains.conversation.memory import ConversationalBufferWindowMemory

a = ConversationalBufferWindowMemory(k=10)



# doc is here https://platform.openai.com/docs/guides/chat/chat-vs-completions?utm_medium=email&_hsmi=248334739&utm_content=248334739&utm_source=hs_email
# chat_completion = openai.ChatCompletion.create(
#     model="gpt-3.5-turbo",
#     api_key="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",
#     messages=[
#         # system message first, it helps set the behavior of the assistant
#         {"role": "system", "content": "You are a helpful assistant."},
#         # I am the user, and this is my prompt
#         {"role": "user", "content": "What's the best star wars movie?"},
#         # we can also add the previous conversation
#         # {"role": "assistant", "content": "Episode III."},
#     ],
# )
#
# print(chat_completion.choices[0].message.content)


from ailab.atp_finetuner.model.model import AILabModel
from transformers import ViltConfig, ViltForQuestionAnswering
from ailab.atp_finetuner.build import ModelRg
from ailab.atp_finetuner.constant import Task, Model

@ModelRg.register((Task.visual_question_answering, Model.vilt))
class ViltForVQAModel(AILabModel):
    def __init__(self, model: any) -> None:
        super().__init__(model)

    def forward(self):
        pass
    
    @classmethod
    def build_model(cls, device_name:str, model_name:str, model_dir:str, **kwargs):
        model_name_or_dir = model_name if model_dir is None else model_dir
        config = ViltConfig.from_pretrained(model_name_or_dir)
        model = ViltForQuestionAnswering.from_pretrained(model_name_or_dir,
                                                    num_labels=len(config.id2label),
                                                    id2label=config.id2label,
                                                    label2id=config.label2id)
        import torch
        device = torch.device(device_name)
        model.to(device)
        return cls(model)
    
    def get_inside_models(self, model_type:str):
        pass



import json
import os
import time

import pandas as pd
from tqdm import tqdm
from ailab.atp_evaluation.benchmarks.base import AILabBenchmark
from ailab.atp_evaluation.build import BenchmarkRg
from ailab.atp_evaluation.constant import BenchMarkType
from ailab.atp_evaluation.models.base import AILabModel
from ailab.log import logger

subject_list = {
    "high_school_physics": "高中物理",
    "fire_engineer": "注册消防工程师",
    "computer_network": "计算机网络",
    "advanced_mathematics": "高等数学",
    "logic": "逻辑学",
    "middle_school_physics": "初中物理",
    "clinical_medicine": "临床医学",
    "probability_and_statistics": "概率统计",
    "ideological_and_moral_cultivation": "思想道德修养与法律基础",
    "operating_system": "操作系统",
    "middle_school_mathematics": "初中数学",
    "chinese_language_and_literature": "中国语言文学",
    "electrical_engineer": "注册电气工程师",
    "business_administration": "工商管理",
    "high_school_geography": "高中地理",
    "modern_chinese_history": "近代史纲要",
    "legal_professional": "法律职业资格",
    "middle_school_geography": "初中地理",
    "middle_school_chemistry": "初中化学",
    "high_school_biology": "高中生物",
    "high_school_chemistry": "高中化学",
    "physician": "医师资格",
    "high_school_chinese": "高中语文",
    "tax_accountant": "税务师",
    "high_school_history": "高中历史",
    "mao_zedong_thought": "毛泽东思想和中国特色社会主义理论概论",
    "high_school_mathematics": "高中数学",
    "professional_tour_guide": "导游资格",
    "veterinary_medicine": "兽医学",
    "environmental_impact_assessment_engineer": "环境影响评价工程师",
    "basic_medicine": "基础医学",
    "education_science": "教育学",
    "urban_and_rural_planner": "注册城乡规划师",
    "middle_school_biology": "初中生物",
    "plant_protection": "植物保护",
    "middle_school_history": "初中历史",
    "high_school_politics": "高中政治",
    "metrology_engineer": "注册计量师",
    "art_studies": "艺术学",
    "college_economics": "大学经济学",
    "college_chemistry": "大学化学",
    "law": "法学",
    "sports_science": "体育学",
    "civil_servant": "公务员",
    "college_programming": "大学编程",
    "middle_school_politics": "初中政治",
    "teacher_qualification": "教师资格",
    "computer_architecture": "计算机组成",
    "college_physics": "大学物理",
    "discrete_mathematics": "离散数学",
    "marxism": "马克思主义基本原理",
    "accountant": "注册会计师",
}

@BenchmarkRg.register(BenchMarkType.ceval)
class CEVALBenchmark(AILabBenchmark):
    def __init__(self, model: AILabModel, dataset_dir: str, ntrain: int, output_dir: str, **kwargs) -> None:
        super().__init__()
        outpath = os.path.join(output_dir, r"logs_ceval")
        if not os.path.exists(outpath):
            os.mkdir(outpath)
        run_date = time.strftime('%Y-%m-%d_%H-%M-%S', time.localtime(time.time()))
        logger.info(model.model_name)
        result_dir_prefix = model.model_name.replace("/", "_")
        self._result_dir = os.path.join(outpath, f"{result_dir_prefix}_{run_date}")
        os.mkdir(self._result_dir)
        if not os.path.exists(dataset_dir):
            raise TypeError(f'{dataset_dir} is not exist')
        self._dataset_dir = dataset_dir
        self._kshot = ntrain
        self._model = model
        self._result_file = f'ceval_{result_dir_prefix}_{ntrain}_shot.json'
        
    def evaluate(self):
        results = dict()
        for sub_name in list(sorted(subject_list.keys())):
            logger.info(f"start eval {sub_name}")
            test_file_name = os.path.join(f'{self._dataset_dir}/test', f'{sub_name}_test.csv')
            logger.info(test_file_name)
            test_df = pd.read_csv(test_file_name)
            dev_df = None
            if self._kshot:
                dev_file_name = os.path.join(f'{self._dataset_dir}/dev', f'{sub_name}_dev.csv')
                dev_df = pd.read_csv(dev_file_name)
            sub_results = self.__eval_subject(sub_name, test_df, dev_df, self._kshot)
            results.update({sub_name:sub_results})
            json.dump(results, open(os.path.join(self._result_dir, self._result_file), 'w'), indent=True, ensure_ascii=False)
    
    def __eval_subject(self, subject_name, test_df, dev_df=None, kshot=-1):
        results = {}
        train_prompt = self.__gen_few_shot_prompt(dev_df, subject_name, kshot)
        for i in tqdm(range(test_df.shape[0])):
            question = self.__format_example(test_df, i, include_answer=False)
            prompt = train_prompt + question
            # logger.info(prompt)
            ans = self._model.get_answer_of_multiple_choices_question(prompt)
            # logger.info(ans)
            idx = test_df.iloc[i]['id']
            results[str(idx)] = ans
        return results
    
    def __format_example(self, df, idx, include_answer=True):
        prompt = df.iloc[idx]['question']
        for choice in self._choices:
            option_data = df.iloc[idx][choice]
            prompt += f'\n{choice}. {option_data}'
        prompt += "\nAnswer:"
        if include_answer:
            prompt += " {}\n\n".format(df.iloc[idx]['answer'])
        return prompt

    def __gen_few_shot_prompt(self, train_df, subject, k=-1):
        prompt = "The following is multiple choice question about {}.\n\n".format(
            self._format_subject(subject)
        )
        if k == -1:
            k = train_df.shape[0]
        if k > 0:
            prompt = "The following are multiple choice questions (with answers) about {}.\n\n".format(
                self._format_subject(subject)
            )
        for i in range(k):
            prompt += self.__format_example(train_df, i)
        return prompt



import argparse
from dataclasses import dataclass
import datetime

import json
import os
from typing import Dict, List, OrderedDict, Tuple
import numpy as np

METRICS = ["acc_norm", "acc_norm", "acc", "mc2", "acc_norm", "acc_norm"]
BENCHMARKS = ["arc_challenge", "hellaswag", "hendrycksTest", "truthfulqa_mc", "Ceval-valid", "Cmmlu"]
BENCH_TO_NAME = {
    "arc_challenge": "ARC",
    "hellaswag": "HellaSwag",
    "hendrycksTest": "MMLU",
    "truthfulqa_mc": "TruthfulQA",
    "Ceval-valid":"CEVAL",
    "Cmmlu":"CMMLU",
}

@dataclass
class EvalResult:
    eval_name: str
    model_args: str
    results: dict

    def to_dict(self):
        data_dict = {}

        data_dict["eval_name"] = self.eval_name # not a column, just a save name
        data_dict["model_args"] = self.model_args
        for benchmark in BENCHMARKS:
            if benchmark not in self.results.keys():
                self.results[benchmark] = None

        for k, v in BENCH_TO_NAME.items():
            data_dict[v] = self.results[k]

        return data_dict

def parse_eval_result(json_filepath: str) -> Tuple[str, list[dict]]:
    with open(json_filepath) as fp:
        data = json.load(fp)

    
    for mmlu_k in ["harness|hendrycksTest-abstract_algebra|5", "hendrycksTest-abstract_algebra"]:
        if mmlu_k in data["versions"] and data["versions"][mmlu_k] == 0:
            return None, [] # we skip models with the wrong version 

    config = data["config"]
    model_args = config.get("model_args", None)

    params_list = model_args.split(',') 
    model_params_dict = {}
    for param in params_list:
        key, value = param.split('=', 1)
        model_params_dict[key.strip()] = value.strip("'")

    # print(model_params_dict)
    pretrained_str = model_params_dict["pretrained"]
    pretrained_model = pretrained_str.split('/')[-1]
    peft_str = model_params_dict.get("peft", None)

    result_key = pretrained_model
    if peft_str is not None:
        peft_name = peft_str.split('/')[-1]
        result_key = f"{pretrained_model}--{peft_name}"

    # print(result_key)
    eval_results = []
    for benchmark, metric in zip(BENCHMARKS, METRICS):
        accs = np.array([v[metric] for k, v in data["results"].items() if benchmark in k])
        if accs.size == 0:
            continue
        mean_acc = np.mean(accs) * 100.0
        eval_results.append(EvalResult(
            eval_name=result_key, model_args=model_args, results={benchmark: mean_acc} 
        ))
    # print(eval_results)
    return result_key, eval_results

def sort_files_by_created_time(files):

    # 获取每个文件元数据中的创建时间
    file_stats = [os.stat(file) for file in files]  
    create_times = [datetime.datetime.fromtimestamp(stat.st_ctime) for stat in file_stats]

    # 按创建时间进行排序
    sorted_paths = [x for _,x in sorted(zip(create_times,files))] 
    
    return sorted_paths

def get_eval_results(dir:str) -> List[EvalResult]:
    json_filepaths = []

    for filename in os.listdir(dir):
        if filename.endswith('.json'):
            json_filepaths.append(os.path.join(dir, filename))
    json_filepaths = sort_files_by_created_time(json_filepaths)

    print("total:", len(json_filepaths), ", list:", json_filepaths)

    eval_results = {}
    for json_filepath in json_filepaths:
        result_key, results = parse_eval_result(json_filepath)
        for eval_result in results:
            if result_key in eval_results.keys():
                # print(result_key)
                # print(eval_result.results)
                eval_results[result_key].results.update(eval_result.results)
                eval_results[result_key].model_args = eval_result.model_args
            else:
                eval_results[result_key] = eval_result

    eval_results = [v for v in eval_results.values()]

    return eval_results


def get_eval_results_dicts(dir:str) -> List[Dict]:
    eval_results = get_eval_results(dir)

    return [e.to_dict() for e in eval_results]

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument("--dir", default="./result")
    args = parser.parse_args()

    r = get_eval_results_dicts(args.dir)
    r.sort(key=lambda x: x["eval_name"])

    print(json.dumps(r, indent=2))


# YOLOv5 🚀 by Ultralytics, GPL-3.0 license
"""
Perform test request
"""

import pprint

import requests

DETECTION_URL = 'http://localhost:5000/v1/object-detection/yolov5s'
IMAGE = 'zidane.jpg'

# Read image
with open(IMAGE, 'rb') as f:
    image_data = f.read()

response = requests.post(DETECTION_URL, files={'image': image_data}).json()

pprint.pprint(response)



from transformers import ViltProcessor, ViltConfig
from ailab.atp_dataset.dataset import AILabDataset
from ailab.atp_finetuner.preprossor import AILabPreprocessor
from ailab.atp_finetuner.build import PreProcessorRg
from ailab.atp_finetuner.constant import Task, Model

import torch
from PIL import Image
class VQADataset(torch.utils.data.Dataset):
    """VQA (v2) dataset."""

    def __init__(self, config, questions, annotations, images, processor):
        self.questions = questions
        self.annotations = annotations
        self.processor = processor
        self.config = config
        self.images = images

    def __len__(self):
        return len(self.annotations)

    def __getitem__(self, idx):
        # get image + text
        annotation = self.annotations[idx]
        image = self.images[idx]
        text = self.questions[idx]
        
        encoding = self.processor(image, text, padding="max_length", truncation=True, return_tensors="pt")
        # remove batch dimension
        for k,v in encoding.items():
          encoding[k] = v.squeeze()
        # add labels
        labels = annotation['labels']
        scores = annotation['scores']
        # based on: https://github.com/dandelin/ViLT/blob/762fd3975c180db6fc88f577cf39549983fa373a/vilt/modules/objectives.py#L301
        targets = torch.zeros(len(self.config.id2label))
        for label, score in zip(labels, scores):
              targets[label] = score
        encoding["labels"] = targets

        return encoding

@PreProcessorRg.register((Task.visual_question_answering, Model.vilt))
class ViltPreProcessor(AILabPreprocessor):
    def __init__(self, dataset, preprocessor):
        self.config = None
        super().__init__(dataset, preprocessor)
    
    @classmethod
    def build_preprocessor(cls, model_name:str, dataset: AILabDataset,pc_dir:str,**kwargs):
        pc_name_dir = model_name if pc_dir is None else pc_dir
        config = ViltConfig.from_pretrained(pc_name_dir)
        hf_preprocessor = ViltProcessor.from_pretrained(pc_name_dir)
        vilt_ins =  cls(dataset, hf_preprocessor)
        vilt_ins.config = config
        return vilt_ins
    
    def process_data(self) ->AILabDataset:
        processor = self._preprocessor
        dataset = self._dataset.to_hf_dataset()["train"]
        answers = dataset["answers_original"]
        annotations = []
        config = self.config
        def get_score(count: int) -> float:
            return min(1.0, count / 3)

        for answer_list in answers:
            answer_count = {}
            for answer in answer_list:
                answer_ = answer["answer"]
                answer_count[answer_] = answer_count.get(answer_, 0) + 1
            labels = []
            scores = []
            for answer in answer_count:
                if answer not in list(config.label2id.keys()):
                    continue
                labels.append(config.label2id[answer])
                score = get_score(answer_count[answer])
                scores.append(score)

            annotation = {}
            annotation['labels'] = labels
            annotation['scores'] = scores
            annotations.append(annotation)

        vqadataset = VQADataset(config = self.config,
                     questions=dataset["question"][:100],
                     annotations=annotations[:100],
                     images = dataset["image"][:100],
                     processor=processor)
        return AILabDataset(vqadataset)



# YOLOv5 🚀 by Ultralytics, GPL-3.0 license
"""
Validate a trained YOLOv5 classification model on a classification dataset

Usage:
    $ bash data/scripts/get_imagenet.sh --val  # download ImageNet val split (6.3G, 50000 images)
    $ python classify/val.py --weights yolov5m-cls.pt --data ../datasets/imagenet --img 224  # validate ImageNet

Usage - formats:
    $ python classify/val.py --weights yolov5s-cls.pt                 # PyTorch
                                       yolov5s-cls.torchscript        # TorchScript
                                       yolov5s-cls.onnx               # ONNX Runtime or OpenCV DNN with --dnn
                                       yolov5s-cls_openvino_model     # OpenVINO
                                       yolov5s-cls.engine             # TensorRT
                                       yolov5s-cls.mlmodel            # CoreML (macOS-only)
                                       yolov5s-cls_saved_model        # TensorFlow SavedModel
                                       yolov5s-cls.pb                 # TensorFlow GraphDef
                                       yolov5s-cls.tflite             # TensorFlow Lite
                                       yolov5s-cls_edgetpu.tflite     # TensorFlow Edge TPU
                                       yolov5s-cls_paddle_model       # PaddlePaddle
"""

import argparse
import os
import sys
from pathlib import Path

import torch
from tqdm import tqdm

FILE = Path(__file__).resolve()
ROOT = FILE.parents[1]  # YOLOv5 root directory
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH
ROOT = Path(os.path.relpath(ROOT, Path.cwd()))  # relative

from models.common import DetectMultiBackend
from utils.dataloaders import create_classification_dataloader
from utils.general import (LOGGER, TQDM_BAR_FORMAT, Profile, check_img_size, check_requirements, colorstr,
                           increment_path, print_args)
from utils.torch_utils import select_device, smart_inference_mode


@smart_inference_mode()
def run(
    data=ROOT / '../datasets/mnist',  # dataset dir
    weights=ROOT / 'yolov5s-cls.pt',  # model.pt path(s)
    batch_size=128,  # batch size
    imgsz=224,  # inference size (pixels)
    device='',  # cuda device, i.e. 0 or 0,1,2,3 or cpu
    workers=8,  # max dataloader workers (per RANK in DDP mode)
    verbose=False,  # verbose output
    project=ROOT / 'runs/val-cls',  # save to project/name
    name='exp',  # save to project/name
    exist_ok=False,  # existing project/name ok, do not increment
    half=False,  # use FP16 half-precision inference
    dnn=False,  # use OpenCV DNN for ONNX inference
    model=None,
    dataloader=None,
    criterion=None,
    pbar=None,
):
    # Initialize/load model and set device
    training = model is not None
    if training:  # called by train.py
        device, pt, jit, engine = next(model.parameters()).device, True, False, False  # get model device, PyTorch model
        half &= device.type != 'cpu'  # half precision only supported on CUDA
        model.half() if half else model.float()
    else:  # called directly
        device = select_device(device, batch_size=batch_size)

        # Directories
        save_dir = increment_path(Path(project) / name, exist_ok=exist_ok)  # increment run
        save_dir.mkdir(parents=True, exist_ok=True)  # make dir

        # Load model
        model = DetectMultiBackend(weights, device=device, dnn=dnn, fp16=half)
        stride, pt, jit, engine = model.stride, model.pt, model.jit, model.engine
        imgsz = check_img_size(imgsz, s=stride)  # check image size
        half = model.fp16  # FP16 supported on limited backends with CUDA
        if engine:
            batch_size = model.batch_size
        else:
            device = model.device
            if not (pt or jit):
                batch_size = 1  # export.py models default to batch-size 1
                LOGGER.info(f'Forcing --batch-size 1 square inference (1,3,{imgsz},{imgsz}) for non-PyTorch models')

        # Dataloader
        data = Path(data)
        test_dir = data / 'test' if (data / 'test').exists() else data / 'val'  # data/test or data/val
        dataloader = create_classification_dataloader(path=test_dir,
                                                      imgsz=imgsz,
                                                      batch_size=batch_size,
                                                      augment=False,
                                                      rank=-1,
                                                      workers=workers)

    model.eval()
    pred, targets, loss, dt = [], [], 0, (Profile(), Profile(), Profile())
    n = len(dataloader)  # number of batches
    action = 'validating' if dataloader.dataset.root.stem == 'val' else 'testing'
    desc = f'{pbar.desc[:-36]}{action:>36}' if pbar else f'{action}'
    bar = tqdm(dataloader, desc, n, not training, bar_format=TQDM_BAR_FORMAT, position=0)
    with torch.cuda.amp.autocast(enabled=device.type != 'cpu'):
        for images, labels in bar:
            with dt[0]:
                images, labels = images.to(device, non_blocking=True), labels.to(device)

            with dt[1]:
                y = model(images)

            with dt[2]:
                pred.append(y.argsort(1, descending=True)[:, :5])
                targets.append(labels)
                if criterion:
                    loss += criterion(y, labels)

    loss /= n
    pred, targets = torch.cat(pred), torch.cat(targets)
    correct = (targets[:, None] == pred).float()
    acc = torch.stack((correct[:, 0], correct.max(1).values), dim=1)  # (top1, top5) accuracy
    top1, top5 = acc.mean(0).tolist()

    if pbar:
        pbar.desc = f'{pbar.desc[:-36]}{loss:>12.3g}{top1:>12.3g}{top5:>12.3g}'
    if verbose:  # all classes
        LOGGER.info(f"{'Class':>24}{'Images':>12}{'top1_acc':>12}{'top5_acc':>12}")
        LOGGER.info(f"{'all':>24}{targets.shape[0]:>12}{top1:>12.3g}{top5:>12.3g}")
        for i, c in model.names.items():
            acc_i = acc[targets == i]
            top1i, top5i = acc_i.mean(0).tolist()
            LOGGER.info(f'{c:>24}{acc_i.shape[0]:>12}{top1i:>12.3g}{top5i:>12.3g}')

        # Print results
        t = tuple(x.t / len(dataloader.dataset.samples) * 1E3 for x in dt)  # speeds per image
        shape = (1, 3, imgsz, imgsz)
        LOGGER.info(f'Speed: %.1fms pre-process, %.1fms inference, %.1fms post-process per image at shape {shape}' % t)
        LOGGER.info(f"Results saved to {colorstr('bold', save_dir)}")

    return top1, top5, loss


def parse_opt():
    parser = argparse.ArgumentParser()
    parser.add_argument('--data', type=str, default=ROOT / '../datasets/mnist', help='dataset path')
    parser.add_argument('--weights', nargs='+', type=str, default=ROOT / 'yolov5s-cls.pt', help='model.pt path(s)')
    parser.add_argument('--batch-size', type=int, default=128, help='batch size')
    parser.add_argument('--imgsz', '--img', '--img-size', type=int, default=224, help='inference size (pixels)')
    parser.add_argument('--device', default='', help='cuda device, i.e. 0 or 0,1,2,3 or cpu')
    parser.add_argument('--workers', type=int, default=8, help='max dataloader workers (per RANK in DDP mode)')
    parser.add_argument('--verbose', nargs='?', const=True, default=True, help='verbose output')
    parser.add_argument('--project', default=ROOT / 'runs/val-cls', help='save to project/name')
    parser.add_argument('--name', default='exp', help='save to project/name')
    parser.add_argument('--exist-ok', action='store_true', help='existing project/name ok, do not increment')
    parser.add_argument('--half', action='store_true', help='use FP16 half-precision inference')
    parser.add_argument('--dnn', action='store_true', help='use OpenCV DNN for ONNX inference')
    opt = parser.parse_args()
    print_args(vars(opt))
    return opt


def main(opt):
    check_requirements(exclude=('tensorboard', 'thop'))
    run(**vars(opt))


if __name__ == '__main__':
    opt = parse_opt()
    main(opt)



import argparse
import json
from tqdm import tqdm
import transformers
import datasets
from ailab.atp_finetuner.preprossor import AILabPreprocessor
from ailab.atp_dataset.dataset import AILabDataset
from transformers import models
from ailab.atp_finetuner.build import PreProcessorRg
from ailab.atp_finetuner.constant import Task, Model

@PreProcessorRg.register((Task.question_answering, Model.distilbert_base_uncased))
class QuestionAnswerPreProcessor(AILabPreprocessor):
    def __init__(self, dataset, preprocessor):
        super().__init__(dataset, preprocessor)

    @classmethod
    def build_preprocessor(cls, model_name:str, dataset: AILabDataset, pc_dir:str, **kwargs):
        pc_name_dir = model_name if pc_dir is None else pc_dir
        hf_preprocessor = models.auto.AutoProcessor.from_pretrained(pc_name_dir, **kwargs)
        return cls(dataset, hf_preprocessor)

    def process_data(self) ->AILabDataset:
        preprocessor = self._preprocessor
        def preprocessor_func(dataset):
                def preprocess(preprocessor, dataset):
                    questions = [q.strip() for q in dataset["question"]]
                    inputs = preprocessor(
                        questions,
                        dataset["context"],
                        max_length=384,
                        truncation="only_second",
                        return_offsets_mapping=True,
                        padding="max_length",
                    )

                    offset_mapping = inputs.pop("offset_mapping")
                    answers = dataset["answers"]
                    start_positions = []
                    end_positions = []

                    for i, offset in enumerate(offset_mapping):
                        answer = answers[i]
                        start_char = answer["answer_start"][0]
                        end_char = answer["answer_start"][0] + len(answer["text"][0])
                        sequence_ids = inputs.sequence_ids(i)

                        # Find the start and end of the context
                        idx = 0
                        while sequence_ids[idx] != 1:
                            idx += 1
                        context_start = idx
                        while sequence_ids[idx] == 1:
                            idx += 1
                        context_end = idx - 1

                        # If the answer is not fully inside the context, label it (0, 0)
                        if offset[context_start][0] > end_char or offset[context_end][1] < start_char:
                            start_positions.append(0)
                            end_positions.append(0)
                        else:
                            # Otherwise it's the start and end token positions
                            idx = context_start
                            while idx <= context_end and offset[idx][0] <= start_char:
                                idx += 1
                            start_positions.append(idx - 1)

                            idx = context_end
                            while idx >= context_start and offset[idx][1] >= end_char:
                                idx -= 1
                            end_positions.append(idx + 1)

                    inputs["start_positions"] = start_positions
                    inputs["end_positions"] = end_positions
                    return inputs
                return preprocess(preprocessor, dataset)
        tokenized_dataset = self._dataset.to_hf_dataset().map(preprocessor_func, batched=True)
        return AILabDataset(tokenized_dataset)



from typing import List
import torch
from transformers import AutoModelForCausalLM,AutoConfig, BitsAndBytesConfig
from peft import TaskType,LoraConfig,get_peft_model
from ailab.atp_finetuner.model.model import AILabModel
from ailab.atp_finetuner.build import ModelRg
from ailab.atp_finetuner.constant import Task, Model
from ailab.log import logger

@ModelRg.register((Task.question_answering, Model.baichuan_7b))
@ModelRg.register((Task.question_answering, Model.baichuan_13b))
@ModelRg.register((Task.question_answering, Model.bloomz_7b1_mt))
@ModelRg.register((Task.question_answering, Model.falcon_7b))
@ModelRg.register((Task.question_answering, Model.moss_moon_003_base))
@ModelRg.register((Task.question_answering, Model.llama2_7b))
@ModelRg.register((Task.question_answering, Model.internlm_7b))
@ModelRg.register((Task.question_answering, Model.belle_7b_2m))
class BaichuanModel(AILabModel):
    def __init__(self, model: any) -> None:
        self.model_name = None
        super().__init__(model)

    def forward(self):
        model = self.model_ins
        output_embedding_layer_name = "lm_head"
        layer_norm_names = ["norm", "ln_f", "ln_attn", "ln_mlp"]
        for name, param in model.named_parameters():
            if param.ndim == 1 and any(layer_norm_name in name for layer_norm_name in layer_norm_names):
                param.data = param.data.to(torch.float32)

        if hasattr(model, "enable_input_require_grads"):
            model.enable_input_require_grads()
        else:
            def make_inputs_require_grad(module, input, output):
                output.requires_grad_(True)
            model.get_input_embeddings().register_forward_hook(make_inputs_require_grad)

        model.gradient_checkpointing_enable()
        model.config.use_cache = False # turn off when gradient checkpointing is enabled

        if hasattr(model, output_embedding_layer_name):
            output_embedding_layer: torch.nn.Linear = getattr(model, output_embedding_layer_name)
            input_dtype = output_embedding_layer.weight.dtype

            class CastOutputToFloat(torch.nn.Sequential):
                def forward(self, x: torch.Tensor) -> torch.Tensor:
                    return super().forward(x.to(input_dtype)).to(torch.float32)

            setattr(model, output_embedding_layer_name, CastOutputToFloat(output_embedding_layer))


        target_modules_dict = {
            Model.baichuan_7b : ['W_pack'],
            Model.baichuan_13b : ['W_pack'],
            Model.bloomz_7b1_mt : ['query_key_value'],
            Model.falcon_7b : ['query_key_value'],
            Model.moss_moon_003_base : ['q_proj','v_proj'],
            Model.llama2_7b : ['q_proj','v_proj'],
            Model.internlm_7b: ['q_proj','v_proj'],
            Model.belle_7b_2m: ['query_key_value']
        }
        
        lora_config = LoraConfig(
                task_type=TaskType.CAUSAL_LM,
                inference_mode=False,
                r=8,
                lora_alpha=32.0,
                lora_dropout=0.1,
                target_modules=target_modules_dict.get(self.model_name),
            )
        model = get_peft_model(model, lora_config)

        def print_trainable_params(model: torch.nn.Module) -> None:
            trainable_params, all_param = 0, 0
            for param in model.parameters():
                num_params = param.numel()
                # if using DS Zero 3 and the weights are initialized empty
                if num_params == 0 and hasattr(param, "ds_numel"):
                    num_params = param.ds_numel
                all_param += num_params
                if param.requires_grad:
                    trainable_params += num_params
            logger.info("trainable params: {:d} || all params: {:d} || trainable%: {:.4f}".format(
                        trainable_params, all_param, 100 * trainable_params / all_param))
        print_trainable_params(model)
        self._model = model
    
    @classmethod
    def build_model(cls, device_name:str, model_name:str, model_dir:str, **kwargs):
        # quantization args
        quantization_bit = kwargs.get('quantization_bit', None)
        quantization_type = kwargs.get('quantization_type', 'nf4')
        double_quantization = kwargs.get("double_quantization", True)
        compute_dtype = kwargs.get('compute_dtype', torch.float16)
        
        #qlora 的配置参数
        config_kwargs = {"trust_remote_code": True} 

        model_name_or_dir = model_name if model_dir is None else model_dir
        config = AutoConfig.from_pretrained(model_name_or_dir, trust_remote_code=True)
        # Quantization configurations (using bitsandbytes library).
        if quantization_bit is not None:
            if quantization_bit == 8:
                config_kwargs["load_in_8bit"] = True
                config_kwargs["quantization_config"] = BitsAndBytesConfig(
                    load_in_8bit=True,
                )

            elif quantization_bit == 4:
                config_kwargs["load_in_4bit"] = True
                config_kwargs["quantization_config"] = BitsAndBytesConfig(
                    load_in_4bit=True,
                    bnb_4bit_compute_dtype=compute_dtype,
                    bnb_4bit_use_double_quant=double_quantization,
                    bnb_4bit_quant_type=quantization_type
                )

            logger.info("Quantizing model to {} bit.".format(quantization_bit))
            
        model = AutoModelForCausalLM.from_pretrained(
            model_name_or_dir,
            config=config,
            torch_dtype=torch.bfloat16 if compute_dtype == torch.bfloat16 else torch.float16,#QLoRA
            low_cpu_mem_usage=True,
            **config_kwargs,
        )

        if hasattr(config, "auto_map") and "AutoConfig" in config.auto_map:
            config.__class__.register_for_auto_class()
        if hasattr(config, "auto_map") and "AutoModelForCausalLM" in config.auto_map:
            model.__class__.register_for_auto_class()
        cls_model =  cls(model)
        cls_model.model_name = model_name
        return cls_model
    
    def get_inside_models(self, model_type:str):
        pass



#!/usr/bin/env python
# coding:utf-8
""" 
@author: nivic ybyang7
@license: Apache Licence 
@file: entrypoint.py
@time: 2023/07/10
@contact: ybyang7@iflytek.com
@site:  
@software: PyCharm 

# code is far away from bugs with the god animal protecting
    I love animals. They taste delicious.
              ┏┓      ┏┓
            ┏┛┻━━━┛┻┓
            ┃      ☃      ┃
            ┃  ┳┛  ┗┳  ┃
            ┃      ┻      ┃
            ┗━┓      ┏━┛
                ┃      ┗━━━┓
                ┃  神兽保佑    ┣┓
                ┃　永无BUG！   ┏┛
                ┗┓┓┏━┳┓┏┛
                  ┃┫┫  ┃┫┫
                  ┗┻┛  ┗┻┛ 
"""

#  Copyright (c) 2022. Lorem ipsum dolor sit amet, consectetur adipiscing elit.
#  Morbi non lorem porttitor neque feugiat blandit. Ut vitae ipsum eget quam lacinia accumsan.
#  Etiam sed turpis ac ipsum condimentum fringilla. Maecenas magna.
#  Proin dapibus sapien vel ante. Aliquam erat volutpat. Pellentesque sagittis ligula eget metus.
#  Vestibulum commodo. Ut rhoncus gravida arcu.
import os
import subprocess
import sys
from ailab.log import logger

def prepare_req():
    req_str = os.environ.get("PIP_REQUIREMENTS", "")
    if not req_str:
        return
    tempfile = '/tmp/requirements.txt'
    req_list = req_str.split(",")
    with open(tempfile, 'w') as temp:
        for line in req_list:
            temp.write(line + "\n")
    temp.close()
    cmds = [
        "pip config set global.index-url https://repo.model.xfyun.cn/api/packages/administrator/pypi/simple  &&  pip config set global.extra-index-url https://pypi.mirrors.ustc.edu.cn/simple/",
        'pip install  -r  %s' % tempfile,

    ]
    for cm in cmds:
        subprocess.call(cm, shell=True)


def prepare_system():
    req_str = os.environ.get("APT_PACKAGES", "")
    if not req_str:
        return
    req_list = ' '.join(req_str.split(","))
    cmds = [
        'apt update',
        'apt install  -y  %s' % req_list,
    ]
    for cm in cmds:
        subprocess.call(cm, shell=True)


DEFAULT_CODE_DIR = '/work/'
USER_SCRIPT = "run.py"


def prepare_script():
    script_path = os.environ.get("SCRIPT_PATH", "/work/code/run.tar.gz")
    # 约定传来的是 tar.gz
    if not os.path.exists(script_path):
        logger.info(f"Not exists: %s" % script_path)
        sys.exit(-1)
    if not script_path.endswith('.tar.gz'):
        logger.info(f"script not a tar.gz")
        sys.exit(-1)
    if not os.path.exists(DEFAULT_CODE_DIR):
        os.makedirs(DEFAULT_CODE_DIR)

    unpress = "tar zxvf %s --strip-components 1 -C %s" % (script_path, DEFAULT_CODE_DIR)
    ret = subprocess.call(unpress, shell=True)
    if ret == 0:
        logger.info("Success Unpress %s" % script_path)
    else:
        logger.info("Please Check....")
        sys.exit(ret)

    # check压缩包是否有 run.py
    fpath = os.path.join(DEFAULT_CODE_DIR, USER_SCRIPT)
    if not os.path.exists(fpath):
        logger.info("Not found....%s" % USER_SCRIPT)
        sys.exit(-1)

    return fpath


def main():
    try:
        prepare_req()
        prepare_system()
    except Exception as e:
        logger.info("Warning: some packages install error. ")

    script = prepare_script()

    if not os.path.exists(script):
        logger.info(f"Error: f{script} not exists... check... ")
    elif not script.endswith(".py"):
        logger.info(f"Error: not a python file..")
    else:
        logger.info("current workdir: %s" % os.getcwd())
        ret = subprocess.call(f"python {script}", shell=True)
        if ret != 0:
            logger.info("batch inference execute error..")
            sys.exit(ret)

        logger.info("Successfully Inference Offiline Job Done")


if __name__ == '__main__':
    main()



from typing import Callable
from abc import ABC, abstractmethod
from ailab.atp_dataset.dataset import AILabDataset
from ailab.atp_finetuner.model import AILabModel
from ailab.atp_finetuner.datacollator import AILabDataCollator
from ailab.atp_finetuner.metric import AILabMetric
from ailab.atp_finetuner.preprossor import AILabPreprocessor
from ailab.atp_finetuner.constant import Task,Model
from ailab.atp_finetuner.build import TrainerRg

class AILabTrainer(ABC):
    def __init__(self):
        pass

    @abstractmethod
    def preprocess(self, dataset:AILabDataset, model:AILabModel, preprocessor: AILabPreprocessor, \
                      data_collator:AILabDataCollator, metric:AILabMetric, train_progess:Callable, **kwargs):
        pass

    @abstractmethod
    def train(self):
        pass
    
    @abstractmethod
    def postprocess(self):
        pass

    @classmethod
    def from_task_model(cls, task:Task, model_name:Model, dataset:AILabDataset, model:AILabModel, preprocessor: AILabPreprocessor, \
                      data_collator:AILabDataCollator, metric:AILabMetric, train_progess:Callable, **kwargs):
        auto_trainer = TrainerRg.get_cls((task, model_name))()
        if auto_trainer is None:
            raise TypeError(f'auto_trainer is None')
        auto_trainer.preprocess(dataset, model, preprocessor, data_collator, metric, train_progess, **kwargs)
        return auto_trainer
    
    @property
    def accelerator(self):
        return self._accelerator

    @accelerator.setter
    def accelerator(self, value):
        self._accelerator = value
        


import enum

class TaskType(enum.Enum):
    NLP = 'nlp'
    AUDIO = 'audio'
    CV = 'cv'
    MULTIMODEL = 'multimodel'

class Task(object):
    text_classification = 'text-classification'
    token_classification = 'token-classification'
    question_answering = 'question-answering'
    question_answering_cn = 'question-answering-cn'
    table_question_answering = 'table-question-answering'
    zero_shot_classification = 'zero-shot-classification'
    translation = 'translation'
    summarization = 'summarization'
    conversational = 'conversational'
    text_generation = 'text-generation'
    text_to_text_generation = 'text-to-text-generation'
    fill_mask = 'fill-mask'

    text_to_speech = 'text-to-speech'
    automatic_speech_recognition = 'automatic-speech-recognition'
    audio_to_audio = 'audio-to-audio'
    audio_classification = 'audio-classification'
    voice_activity_detection = 'voice-activity-detection'

    depth_estimation = 'depth-estimation'
    image_classification = 'image-classification'
    video_calssification = 'video-calssification'
    object_detection = 'object-detection'
    image_segmentation = 'image-segmentation'
    image_to_image = 'image-to-image'
    unconditional_image_generation = 'unconditional-image-generation'
    zero_shot_image_classification = 'zero-shot-image-classification'

    text_to_image = 'text-to-image'
    text_to_video = 'text-to-video'
    feature_extraction = 'feature-extraction'
    image_to_text = 'image-to-text'
    visual_question_answering = 'visual-question-answering'
    document_question_answering = 'document-question-answering'
    graph_machine_learning = 'graph-machine-learning'
    image_captioning = 'image-captioning'

class Model(object):
    chatglm_6b = 'chatglm_6b'
    llama_7b = 'llama_7b'
    alpaca = 'standford_alpaca'
    chinese_alpaca = 'chinese_llama_alpaca'
    vicuna = 'chinese_llama_vicuna'

class BenchMarkType(enum.Enum):
    ceval = 'ceval'
    mmlu = 'mmlu'
    harness = 'harness'


from .datacollator import AILabDataCollator
from .dc_for_token import DataCollatorForToken
from .dc_with_padding import DataCollatorWithPadding
from .dc_for_alpaca import DataCollatorForAlpaca
from .dc_for_language import DataCollatorForLanguage
from .dc_for_chinese_alpaca import ChineseAlpacaCollator
from .dc_for_baichuan import BaichuanDataCollator
from .dc_for_chatglm import ChatglmDataCollator
from .dc_for_ziya_llama_13b import DataCollatorForZiyallama13b



import torch
import torch.nn as nn
import torch.nn.functional as F

from ..general import xywh2xyxy
from ..loss import FocalLoss, smooth_BCE
from ..metrics import bbox_iou
from ..torch_utils import de_parallel
from .general import crop_mask


class ComputeLoss:
    # Compute losses
    def __init__(self, model, autobalance=False, overlap=False):
        self.sort_obj_iou = False
        self.overlap = overlap
        device = next(model.parameters()).device  # get model device
        h = model.hyp  # hyperparameters

        # Define criteria
        BCEcls = nn.BCEWithLogitsLoss(pos_weight=torch.tensor([h['cls_pw']], device=device))
        BCEobj = nn.BCEWithLogitsLoss(pos_weight=torch.tensor([h['obj_pw']], device=device))

        # Class label smoothing https://arxiv.org/pdf/1902.04103.pdf eqn 3
        self.cp, self.cn = smooth_BCE(eps=h.get('label_smoothing', 0.0))  # positive, negative BCE targets

        # Focal loss
        g = h['fl_gamma']  # focal loss gamma
        if g > 0:
            BCEcls, BCEobj = FocalLoss(BCEcls, g), FocalLoss(BCEobj, g)

        m = de_parallel(model).model[-1]  # Detect() module
        self.balance = {3: [4.0, 1.0, 0.4]}.get(m.nl, [4.0, 1.0, 0.25, 0.06, 0.02])  # P3-P7
        self.ssi = list(m.stride).index(16) if autobalance else 0  # stride 16 index
        self.BCEcls, self.BCEobj, self.gr, self.hyp, self.autobalance = BCEcls, BCEobj, 1.0, h, autobalance
        self.na = m.na  # number of anchors
        self.nc = m.nc  # number of classes
        self.nl = m.nl  # number of layers
        self.nm = m.nm  # number of masks
        self.anchors = m.anchors
        self.device = device

    def __call__(self, preds, targets, masks):  # predictions, targets, model
        p, proto = preds
        bs, nm, mask_h, mask_w = proto.shape  # batch size, number of masks, mask height, mask width
        lcls = torch.zeros(1, device=self.device)
        lbox = torch.zeros(1, device=self.device)
        lobj = torch.zeros(1, device=self.device)
        lseg = torch.zeros(1, device=self.device)
        tcls, tbox, indices, anchors, tidxs, xywhn = self.build_targets(p, targets)  # targets

        # Losses
        for i, pi in enumerate(p):  # layer index, layer predictions
            b, a, gj, gi = indices[i]  # image, anchor, gridy, gridx
            tobj = torch.zeros(pi.shape[:4], dtype=pi.dtype, device=self.device)  # target obj

            n = b.shape[0]  # number of targets
            if n:
                pxy, pwh, _, pcls, pmask = pi[b, a, gj, gi].split((2, 2, 1, self.nc, nm), 1)  # subset of predictions

                # Box regression
                pxy = pxy.sigmoid() * 2 - 0.5
                pwh = (pwh.sigmoid() * 2) ** 2 * anchors[i]
                pbox = torch.cat((pxy, pwh), 1)  # predicted box
                iou = bbox_iou(pbox, tbox[i], CIoU=True).squeeze()  # iou(prediction, target)
                lbox += (1.0 - iou).mean()  # iou loss

                # Objectness
                iou = iou.detach().clamp(0).type(tobj.dtype)
                if self.sort_obj_iou:
                    j = iou.argsort()
                    b, a, gj, gi, iou = b[j], a[j], gj[j], gi[j], iou[j]
                if self.gr < 1:
                    iou = (1.0 - self.gr) + self.gr * iou
                tobj[b, a, gj, gi] = iou  # iou ratio

                # Classification
                if self.nc > 1:  # cls loss (only if multiple classes)
                    t = torch.full_like(pcls, self.cn, device=self.device)  # targets
                    t[range(n), tcls[i]] = self.cp
                    lcls += self.BCEcls(pcls, t)  # BCE

                # Mask regression
                if tuple(masks.shape[-2:]) != (mask_h, mask_w):  # downsample
                    masks = F.interpolate(masks[None], (mask_h, mask_w), mode='nearest')[0]
                marea = xywhn[i][:, 2:].prod(1)  # mask width, height normalized
                mxyxy = xywh2xyxy(xywhn[i] * torch.tensor([mask_w, mask_h, mask_w, mask_h], device=self.device))
                for bi in b.unique():
                    j = b == bi  # matching index
                    if self.overlap:
                        mask_gti = torch.where(masks[bi][None] == tidxs[i][j].view(-1, 1, 1), 1.0, 0.0)
                    else:
                        mask_gti = masks[tidxs[i]][j]
                    lseg += self.single_mask_loss(mask_gti, pmask[j], proto[bi], mxyxy[j], marea[j])

            obji = self.BCEobj(pi[..., 4], tobj)
            lobj += obji * self.balance[i]  # obj loss
            if self.autobalance:
                self.balance[i] = self.balance[i] * 0.9999 + 0.0001 / obji.detach().item()

        if self.autobalance:
            self.balance = [x / self.balance[self.ssi] for x in self.balance]
        lbox *= self.hyp['box']
        lobj *= self.hyp['obj']
        lcls *= self.hyp['cls']
        lseg *= self.hyp['box'] / bs

        loss = lbox + lobj + lcls + lseg
        return loss * bs, torch.cat((lbox, lseg, lobj, lcls)).detach()

    def single_mask_loss(self, gt_mask, pred, proto, xyxy, area):
        # Mask loss for one image
        pred_mask = (pred @ proto.view(self.nm, -1)).view(-1, *proto.shape[1:])  # (n,32) @ (32,80,80) -> (n,80,80)
        loss = F.binary_cross_entropy_with_logits(pred_mask, gt_mask, reduction='none')
        return (crop_mask(loss, xyxy).mean(dim=(1, 2)) / area).mean()

    def build_targets(self, p, targets):
        # Build targets for compute_loss(), input targets(image,class,x,y,w,h)
        na, nt = self.na, targets.shape[0]  # number of anchors, targets
        tcls, tbox, indices, anch, tidxs, xywhn = [], [], [], [], [], []
        gain = torch.ones(8, device=self.device)  # normalized to gridspace gain
        ai = torch.arange(na, device=self.device).float().view(na, 1).repeat(1, nt)  # same as .repeat_interleave(nt)
        if self.overlap:
            batch = p[0].shape[0]
            ti = []
            for i in range(batch):
                num = (targets[:, 0] == i).sum()  # find number of targets of each image
                ti.append(torch.arange(num, device=self.device).float().view(1, num).repeat(na, 1) + 1)  # (na, num)
            ti = torch.cat(ti, 1)  # (na, nt)
        else:
            ti = torch.arange(nt, device=self.device).float().view(1, nt).repeat(na, 1)
        targets = torch.cat((targets.repeat(na, 1, 1), ai[..., None], ti[..., None]), 2)  # append anchor indices

        g = 0.5  # bias
        off = torch.tensor(
            [
                [0, 0],
                [1, 0],
                [0, 1],
                [-1, 0],
                [0, -1],  # j,k,l,m
                # [1, 1], [1, -1], [-1, 1], [-1, -1],  # jk,jm,lk,lm
            ],
            device=self.device).float() * g  # offsets

        for i in range(self.nl):
            anchors, shape = self.anchors[i], p[i].shape
            gain[2:6] = torch.tensor(shape)[[3, 2, 3, 2]]  # xyxy gain

            # Match targets to anchors
            t = targets * gain  # shape(3,n,7)
            if nt:
                # Matches
                r = t[..., 4:6] / anchors[:, None]  # wh ratio
                j = torch.max(r, 1 / r).max(2)[0] < self.hyp['anchor_t']  # compare
                # j = wh_iou(anchors, t[:, 4:6]) > model.hyp['iou_t']  # iou(3,n)=wh_iou(anchors(3,2), gwh(n,2))
                t = t[j]  # filter

                # Offsets
                gxy = t[:, 2:4]  # grid xy
                gxi = gain[[2, 3]] - gxy  # inverse
                j, k = ((gxy % 1 < g) & (gxy > 1)).T
                l, m = ((gxi % 1 < g) & (gxi > 1)).T
                j = torch.stack((torch.ones_like(j), j, k, l, m))
                t = t.repeat((5, 1, 1))[j]
                offsets = (torch.zeros_like(gxy)[None] + off[:, None])[j]
            else:
                t = targets[0]
                offsets = 0

            # Define
            bc, gxy, gwh, at = t.chunk(4, 1)  # (image, class), grid xy, grid wh, anchors
            (a, tidx), (b, c) = at.long().T, bc.long().T  # anchors, image, class
            gij = (gxy - offsets).long()
            gi, gj = gij.T  # grid indices

            # Append
            indices.append((b, a, gj.clamp_(0, shape[2] - 1), gi.clamp_(0, shape[3] - 1)))  # image, anchor, grid
            tbox.append(torch.cat((gxy - gij, gwh), 1))  # box
            anch.append(anchors[a])  # anchors
            tcls.append(c)  # class
            tidxs.append(tidx)
            xywhn.append(torch.cat((gxy, gwh), 1) / gain[2:6])  # xywh normalized

        return tcls, tbox, indices, anch, tidxs, xywhn



from typing import List
import torch
from transformers import LlamaForCausalLM,AutoConfig, BitsAndBytesConfig
from ailab.atp_finetuner.model.model import AILabModel
from ailab.atp_finetuner.build import ModelRg
from ailab.atp_finetuner.constant import Task, Model

@ModelRg.register((Task.question_answering, Model.chinese_alpaca))
class ChineseAlpacaModel(AILabModel):
    def __init__(self, model: any) -> None:
        super().__init__(model)

    def forward(self):
        pass
    
    @classmethod
    def build_model(cls, device_name:str, model_name:str, model_dir:str, **kwargs):
        # quantization args
        quantization_bit = kwargs.get('quantization_bit',  None)
        quantization_type = kwargs.get('quantization_type', 'nf4')
        double_quantization = kwargs.get("double_quantization", True)
        compute_dtype = kwargs.get('compute_dtype', torch.float16)
        
        config_kwargs = {} 

        model_name_or_dir = model_name if model_dir is None else model_dir

        config = AutoConfig.from_pretrained(model_name_or_dir)
        quantization_bit = None # Temporarily disabling qlora for erroneous inference results
        # Quantization configurations (using bitsandbytes library).
        if quantization_bit is not None:
            if quantization_bit == 8:
                config_kwargs["load_in_8bit"] = True
                config_kwargs["quantization_config"] = BitsAndBytesConfig(
                    load_in_8bit=True,
                )

            elif quantization_bit == 4:
                config_kwargs["load_in_4bit"] = True
                config_kwargs["quantization_config"] = BitsAndBytesConfig(
                    load_in_4bit=True,
                    bnb_4bit_compute_dtype=compute_dtype,
                    bnb_4bit_use_double_quant=double_quantization,
                    bnb_4bit_quant_type=quantization_type
                )

            logger.info("Quantizing model to {} bit.".format(quantization_bit))

        model = LlamaForCausalLM.from_pretrained(
            model_name_or_dir,
            from_tf=bool(".ckpt" in model_name_or_dir),
            config=config,
            torch_dtype=torch.float16,
            low_cpu_mem_usage=True,
            **config_kwargs
        )
        return cls(model)
    
    def get_inside_models(self, model_type:str):
        pass
